/*
 *    dialog-object-properties.cc
 *
 *    Copyright (C) 1998, 1999  by Juan Pablo Romero Mndez
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    COPYNG file for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **/

#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>

#include <strstream>
#include <iomanip>

#include "global.h"
#include "support.h"
#include "superficie.h"
#include "windows.h"
#include "dialog-scene-properties.h"
#include "dialog-object-properties.h"
#include "gnome-sup.h"


#if HAVE_CONFIG_H
        #include <config.h>
#endif

static GtkWidget *    _ctree;
static GtkCTreeNode * _parent        = NULL;

void
on_show_normal_vectors_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
    (*TheScene)->normals(togglebutton->active);
    TheScene.redraw();
}


void
on_show_frame_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
    (*TheScene)->show_bbox(togglebutton->active);
    TheScene.redraw();
}



void
on_smooth_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
    (*TheScene)->smooth(togglebutton->active);
    TheScene.redraw();
}


void
on_button_down_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
    TheScene.button_down(togglebutton->active);
}

void
on_center_changed(GtkEditable     *editable,
                  gpointer         user_data)
{
    gfloat data[3];
    BaseObject* ob = TheScene.selected_object();
    istrstream entrada(gtk_entry_get_text(GTK_ENTRY(editable)));
    entrada >> data[0] >> data[1] >> data[2];
    ob->center( (gfloat*) data );
    TheScene.redraw();
}

void
on_transparent_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
    (*TheScene)->transparent(togglebutton->active);
    TheScene.redraw();
}


void
on_mesh_toggled                       (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    BaseObject* ob = TheScene.selected_object();

    ob->show_mesh(togglebutton->active);

    /* This obscure the frame, so better update de buttons */
    if(togglebutton->active) {
        ob->smooth_edges(!togglebutton->active);
        update_toggle_button( "smooth_edges", ob->smooth_edges() );
    }


    TheScene.redraw();
}

void
on_mesh_smooth_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    (*TheScene)->mesh_smooth(togglebutton->active);
    TheScene.redraw();
}

void
on_frame_line_width_changed            (GtkSpinButton *spinbutton,
                                        gpointer         user_data)
{
    BaseObject* ob = TheScene.selected_object();
    gfloat val = gtk_spin_button_get_value_as_float(spinbutton);

    ob->frame_line_width(val);

    TheScene.redraw();
}



void
on_smooth_lines_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    (*TheScene)->smooth_lines(togglebutton->active);
    TheScene.redraw();
}


void
on_lines_width_changed                 (GtkSpinButton *spinbutton,
                                        gpointer         user_data)
{
    BaseObject* ob = TheScene.selected_object();
    gfloat val = gtk_spin_button_get_value_as_float(spinbutton);

    ob->lines_width(val);
    TheScene.redraw();
}


void
on_smooth_edges_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    BaseObject* ob = TheScene.selected_object();

    ob->smooth_edges(togglebutton->active);

    /* This obscure the frame, so better update de buttons */
    if(togglebutton->active) {
        ob->show_mesh(!togglebutton->active);
        update_toggle_button( "mesh", ob->show_mesh() );
    }
    TheScene.redraw();
}

void
object_info_update()
{
    if( !info ) return;

    ostrstream array, triangles;

    Object * ob = dynamic_cast<Object*>( TheScene.selected_object() );
    

    if( ob ) {
        array << ob->vertex_size() << " ("
            << ob->vertex_size() * sizeof(float) << " bytes)" << ends;
    }
    else // No es <Object*> ( es <Multiple*> )
        array << "--" << ends;

    if( ob = dynamic_cast<Triangles*>( ob ) )
        triangles << ((Triangles*)ob)->index_size() / 3 << ends;
    else //No es <Triangles*>
        triangles << "--" << ends;

    GtkWidget * label = lookup_widget(info, "info_array_size");
    gtk_label_set_text(GTK_LABEL(label), array.str());

    label = lookup_widget(info, "info_triangles");
    gtk_label_set_text(GTK_LABEL(label), triangles.str());

    //borra el alojamiento
    array.freeze(0);
    triangles.freeze(0);
}

void
on_object_info_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
    if( !info ) {
        //info = glade_xml_get_widget(xml, "info");
        info = create_info();
        object_info_update();
    }

    if( !GTK_WIDGET_VISIBLE (info) )
        gtk_widget_show(info);
}



/* Actualiza los controles de acuerdo al objeto seleccionado.
 * La idea es llamarla cada vez que cambia el objeto seleccionado.
 */

void
update_toggle_button(gchar* widget_name, gboolean val)
{
    /* It is necesary to block the emision of the signal,
     * because the other way the value of the variable gets
     * changed.
     */

    GtkWidget*  widget = lookup_widget(TheTreeW, widget_name);
    gtk_signal_handler_block_by_data(GTK_OBJECT(widget), NULL);
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (widget), val);
    gtk_signal_handler_unblock_by_data(GTK_OBJECT(widget), NULL);
}


void
tree_update(void)
{

    GtkWidget*  widget;
    BaseObject* ob = TheScene.selected_object();

    if( !TheTreeW )
        return;

    update_toggle_button( "smooth",             ob->smooth()       );
    update_toggle_button( "show_normal_vectors",ob->normals()      );
    update_toggle_button( "show_frame",         ob->show_bbox()    );
    update_toggle_button( "transparent",        ob->transparent()  );
    update_toggle_button( "invisible",          ob->invisible  ()  );
    update_toggle_button( "mesh",               ob->show_mesh()    );
    update_toggle_button( "mesh_smooth",        ob->mesh_smooth()  );
    update_toggle_button( "smooth_lines",       ob->smooth_lines() );
    update_toggle_button( "smooth_edges",       ob->smooth_edges() );

    widget = lookup_widget(TheTreeW, "frame_line_width");
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), ob->frame_line_width());

    widget = lookup_widget(TheTreeW, "lines_width");
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), ob->lines_width());

    ostrstream centro;
    centro <<  setiosflags(ios::fixed) << setprecision(2)
        << ob->center()[0] << " "
        << ob->center()[1] << " "
        << ob->center()[2] << ends;

    widget = lookup_widget(TheTreeW, "center");
    gtk_signal_handler_block_by_data(GTK_OBJECT(widget), NULL);
    gtk_entry_set_text( GTK_ENTRY(widget), centro.str() );
    gtk_signal_handler_unblock_by_data(GTK_OBJECT(widget), NULL);

    centro.freeze(0);

    // frame color picker
    widget = lookup_widget(TheTreeW, "frame_colorpicker");
    gnome_color_picker_set_d( GNOME_COLOR_PICKER(widget),
                              ob->frame_color()[0],
                              ob->frame_color()[1],
                              ob->frame_color()[2],
                              ob->frame_color()[3] );

    // normal color picker
    widget = lookup_widget(TheTreeW, "normal_colorpicker");
    gnome_color_picker_set_d( GNOME_COLOR_PICKER(widget),
                              ob->normal_color()[0],
                              ob->normal_color()[1],
                              ob->normal_color()[2],
                              ob->normal_color()[3] );

}

void
on_ctree_tree_select_row(GtkCTree        *ctree,
                         GtkCTreeNode     *node,
                         gint             column,
                         gpointer         user_data)
{
    GNode* gnode = (GNode*) gtk_ctree_node_get_row_data(GTK_CTREE(ctree), node);
    BaseObject* ob = BASE_OBJECT(gnode);
    TheScene.selected_node(gnode);
    tree_update();
    object_info_update();
}

gboolean
on_ctree_key_press_event(GtkWidget       *widget,
                         GdkEventKey     *event,
                         gpointer         user_data)
{

    switch (event->keyval)
    {
    case GDK_Insert:
        app_insert_object(NULL, NULL);
        return TRUE;
        break;
        
    case GDK_Delete:
        app_delete_node(TheScene.selected_node());
        return TRUE;
        break;

    case GDK_Escape:
        gtk_widget_hide(TheTreeW);
        return TRUE;
        break;
    };


    return FALSE;
}


void
on_ctree_tree_move                     (GtkCTree               *ctree,
                                        GtkCTreeNode           *node,
                                        GtkCTreeNode           *new_parent,
                                        GtkCTreeNode           *new_sibling,
                                        gpointer               user_data)
{

    GNode* gnode = (GNode*) gtk_ctree_node_get_row_data(GTK_CTREE(ctree), node);
    GNode* parent = (GNode*) gtk_ctree_node_get_row_data(GTK_CTREE(ctree), new_parent);
    BaseObject* ob = BASE_OBJECT(gnode);

    Multiple* m = (Multiple*) parent->data;

    if( parent ) {
        TheScene.selected_node( m->move(gnode) );
    }
    else {
        g_error("you tried to leave a node without parent");
    }

    //TheScene.selected_node(gnode);
}


void
on_add_group_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
    BaseObject * ob = new Multiple;
    ob->initialize();
    TheScene.append(ob);
    rebuild_tree();
    app_select_node( TheScene.selected_node() );
}


void
on_object_delete_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
    app_delete_node(TheScene.selected_node());
}


void
on_invisible_toggled                   (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    (*TheScene)->invisible(togglebutton->active);
    TheScene.redraw();
}


void
on_frame_color_color_set               (GnomeColorPicker *gnomecolorpicker,
                                        guint            arg1,
                                        guint            arg2,
                                        guint            arg3,
                                        guint            arg4,
                                        gpointer         user_data)
{
    gdouble color[4];

    gnome_color_picker_get_d( gnomecolorpicker,
                              color,
                              color + 1,
                              color + 2,
                              color + 3 );
    (*TheScene)->frame_color(color);
    TheScene.redraw();
}


void
on_normal_color_color_set              (GnomeColorPicker *gnomecolorpicker,
                                        guint            arg1,
                                        guint            arg2,
                                        guint            arg3,
                                        guint            arg4,
                                        gpointer         user_data)
{
    gdouble color[4];

    gnome_color_picker_get_d( gnomecolorpicker,
                              color,
                              color + 1,
                              color + 2,
                              color + 3 );
    (*TheScene)->normal_color(color);
    TheScene.redraw();
}



static void
update_show_as_menu()
{
    GtkWidget * widget;

    widget = lookup_widget(TheTreeW, "button_up_menu");
    gtk_option_menu_set_history(GTK_OPTION_MENU(widget), (*TheScene)->polygon_mode());

}



void
tree_init()
{

    _ctree = lookup_widget(TheTreeW, "ctree");
    gtk_clist_set_reorderable (GTK_CLIST(_ctree), TRUE);
    gtk_ctree_set_expander_style(GTK_CTREE(_ctree), GTK_CTREE_EXPANDER_CIRCULAR);
    gtk_ctree_set_line_style(GTK_CTREE(_ctree), GTK_CTREE_LINES_DOTTED );


    // show as menu

    GtkWidget *widget, *menu, *item;

    gchar * menu_labels[] = {
        N_("Normal"), N_("H-Lines"), N_("V-Lines"), N_("WireFrame"), N_("Points")
    };

    menu = gtk_menu_new();

    for(gint i=0; i < 5; i++) {
        item = make_menu_item(menu_labels[i],
                              GTK_SIGNAL_FUNC(cb_show_as_menu_select),
                              GINT_TO_POINTER(i));
        gtk_menu_append(GTK_MENU (menu), item);
    }

    widget = lookup_widget(TheTreeW, "button_up_menu");
    gtk_option_menu_set_menu(GTK_OPTION_MENU (widget), menu);

}

gboolean
gnode2ctree (GtkCTree     *ctree,
             guint         depth,
             GNode        *gnode,
             GtkCTreeNode *cnode,
             gpointer      data)
{
    gtk_ctree_set_node_info(ctree, cnode, NULL, 2,
                            NULL, NULL, NULL, NULL,
                            FALSE, TRUE);
    gtk_ctree_node_set_text(ctree, cnode, 0, BASE_OBJECT(gnode)->name() );

    gtk_ctree_node_set_row_data(ctree, cnode, gnode);

    return TRUE;
}


void
rebuild_tree()
{
    if( !TheTreeW )
        return;
    
    _ctree = lookup_widget(TheTreeW, "ctree");

    gtk_clist_freeze (GTK_CLIST (_ctree));
    gtk_clist_clear (GTK_CLIST (_ctree));

    gtk_ctree_insert_gnode (GTK_CTREE(_ctree),
                            NULL, NULL,
                            TheScene.gnode(),
                            gnode2ctree,
                            NULL);

    gtk_clist_thaw (GTK_CLIST (_ctree));
}

GtkCTreeNode* ctree_node_found = NULL;

void
find_gtkctreenode_from_gnode(GtkCTree*     ctree,
                             GtkCTreeNode* node,
                             GNode*        gnode)
{
    GNode* gn = (GNode*) gtk_ctree_node_get_row_data(GTK_CTREE (ctree), node);
    if( gn == gnode )
        ctree_node_found = node;
}

void
object_properties_cb (GtkWidget* w, gpointer data)
{

    GtkWidget* ctree;

    if( !TheTreeW ) {
        //TheTreeW = glade_xml_get_widget(xml, "objects");
        TheTreeW = create_objects();
        tree_init();
        tree_update();
        rebuild_tree();

        app_select_node( TheScene.selected_node() );
    }

    if( !GTK_WIDGET_VISIBLE (TheTreeW) )
        gtk_widget_show(TheTreeW);

}

