/* dialog-save-as.cc
 ** Copyright (C) 1999 Juan Pablo Romero <juan_pablo_22@yahoo.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation; either version 2 of the License, or
 ** (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "global.h"
#include "gnome-sup.h"
#include "windows.h"
#include "dialog-scene-properties.h"

#if HAVE_CONFIG_H
#       include <config.h>
#endif

#include <gnome.h>
#include <strstream>

#include <GL/glu.h>
#include <GL/osmesa.h>
#include <GL/glut.h>


static enum { PS, FIG, SKETCH, USE_EXTENSION } save_file_as = PS;

int visual_attributes[] = {
    GDK_GL_RGBA,
    GDK_GL_NONE
};

void
save_as(gchar* file)
{
#if 0 // with gtkglarea
    GdkVisual*      visual;
    GdkGLContext*  context;
    GdkPixmap*      pixmap;
    GdkGLPixmap*  glpixmap;
    GdkImlibImage*   image;

    visual = gdk_gl_choose_visual(visual_attributes);
    if (!visual)
        g_print(_("Can't get visual\n"));

    context = gdk_gl_context_new(visual);
    pixmap = gdk_pixmap_new(NULL, TheScene.window_width(), TheScene.window_height(), visual->depth);
    glpixmap = gdk_gl_pixmap_new(visual, pixmap);

    if (gdk_gl_pixmap_make_current(glpixmap, context)) {
        TheScene.realize();
        TheScene.window_reshape();
        TheScene.draw();
    }

    gdk_gl_context_unref(context);
    gdk_gl_pixmap_unref(glpixmap);

    image = gdk_imlib_create_image_from_drawable(pixmap, NULL, 0, 0,
                                                 TheScene.window_width(),
                                                 TheScene.window_height());
    gdk_imlib_save_image(image, file, NULL);
    gdk_imlib_kill_image(image);
    gdk_pixmap_unref(pixmap);

#else // with mesa

    GdkImlibImage*   image;

    OSMesaContext ctx;
    void *buffer;

    gint width = TheScene.window_width();
    gint height = TheScene.window_height();

    /* Allocate the image buffer */
    buffer = malloc( width * height * 4 );
    if( !buffer ) {
        g_warning(_("coudln't allocate enough space to render the scene"));
        return;
    }

    /* Create an RGBA-mode context */
    ctx = OSMesaCreateContext( OSMESA_RGB, NULL );
    if( !ctx ) {
        free( buffer );
        return;
    }

    /* Bind the buffer to the context and make it current */
    OSMesaMakeCurrent( ctx, buffer, GL_UNSIGNED_BYTE, width, height );

    OSMesaPixelStore( OSMESA_Y_UP, 0 );

    // render the image
    TheScene.realize();
    TheScene.window_reshape();
    TheScene.draw();

    // imlib .........

    image = gdk_imlib_create_image_from_data((unsigned char *) buffer, NULL, width, height);
    gdk_imlib_save_image(image, file, NULL);
    gdk_imlib_kill_image(image);
    free( buffer );
    OSMesaDestroyContext( ctx );
    return;

    // ...............


    // this code is taken from "osdemo.c" by Brian Paul

    FILE *f = fopen( file, "w" );

    if (f) {
        int i, x, y;
        GLubyte *ptr = (GLubyte *) buffer;
        fprintf(f,"P6\n");
        fprintf(f,"# ppm-file created by %s\n",  "superficie");
        fprintf(f,"%i %i\n", width,height);
        fprintf(f,"255\n");
        fclose(f);
        f = fopen( file, "ab" );  /* reopen in binary append mode */
        for( y = height-1; y >= 0; y-- ) {
            for( x = 0; x < width; x++ ) {
                i = (y*width + x) * 4;
                fputc(ptr[i], f);   /* write red */
                fputc(ptr[i+1], f); /* write green */
                fputc(ptr[i+2], f); /* write blue */
            }
        }

        fclose(f);
    }

    /* free the image buffer */
    free( buffer );

    /* destroy the context */
    OSMesaDestroyContext( ctx );

#endif


}



void
on_ok_save_as_fileselection_clicked       (GtkButton       *button, /*FOLD00*/
                                           gpointer         user_data)
{

    gchar * ps_tmp = NULL;
    gchar * file = NULL;
    ostrstream command;

    ps_tmp = tempnam(NULL, "sup");
    file = g_strdup( gtk_file_selection_get_filename(GTK_FILE_SELECTION(user_data)) );

    if( save_file_as == USE_EXTENSION )
        save_as(file);
    //TheScene.save_as(file);
    else
        TheScene.print(ps_tmp);


    switch( save_file_as )
    {
    case PS:
        command << "mv " << ps_tmp << " " << file << ends;
        system(command.str());
        break;

    case FIG:
        command << Pstoedit << " -f fig " << ps_tmp << " " << file << ends;
        system(command.str());
        break;

    case SKETCH:
        command << Pstoedit << " -f sk " << ps_tmp << " " << file << ends;
        system(command.str());
        break;

    case USE_EXTENSION:
        break;
    }

    command.freeze(0);
    g_free(file);
    g_free(ps_tmp);

}

static void
cb_save_ps(GtkWidget * w, gpointer data)
{
    save_file_as = PS;
}

static void
cb_save_xfig(GtkWidget * w, gpointer data)
{
    save_file_as = FIG;
}

static void
cb_save_sketch(GtkWidget * w, gpointer data)
{
    save_file_as = SKETCH;
}

static void
cb_save_use_extension(GtkWidget * w, gpointer data)
{
    save_file_as = USE_EXTENSION;
}


static GtkWidget *
make_format_chooser (void)
{
    GtkWidget *box, *label;
    GtkWidget *omenu, *menu, *item;

    box   = gtk_hbox_new (0, GNOME_PAD);
    label = gtk_label_new (_("File format:"));
    omenu = gtk_option_menu_new ();
    menu  = gtk_menu_new ();

    struct {
        gchar* label;
        void (*callback)(GtkWidget * w, gpointer data);
    } data[] = {
        { N_("PostScript file format"),             cb_save_ps },
        { N_("Xfig file format (via pstoedit)"),    cb_save_xfig },
        { N_("sketch file format  (via pstoedit)"), cb_save_sketch },
        { N_("Determine file type by extension"),   cb_save_use_extension },
        { NULL, NULL }
    };

    for(gint i=0; data[i].label; i++) {
        item = make_menu_item(data[i].label, GTK_SIGNAL_FUNC(data[i].callback), NULL);
        gtk_menu_append(GTK_MENU (menu), item);

    }

    gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu), GTK_WIDGET (menu));

    gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, GNOME_PAD);
    gtk_box_pack_start (GTK_BOX (box), omenu, FALSE, TRUE, GNOME_PAD);
    gtk_widget_show_all (box);

    return box;
}



void
save_as_cb (GtkWidget* w, gpointer data)
{

    static GtkWidget* save_as_fileselection = NULL;
    GtkWidget* format_selector;

    if( !save_as_fileselection ) {
        save_as_fileselection = create_save_as_fileselection();

        format_selector = make_format_chooser ();
        gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION(save_as_fileselection)->action_area), format_selector,
                            FALSE, TRUE, 0);

        // TODO: suggest a name based on the superficie file name
        //gtk_file_selection_set_filename( GTK_FILE_SELECTION(save_as_fileselection), "test.ps" );
    }

    if( !GTK_WIDGET_VISIBLE (save_as_fileselection) )
        gtk_widget_show(save_as_fileselection);
}
