
/*    dialog-scene-properties.cc
 *
 *    Copyright (C) 1998, 1999  by Juan Pablo Romero Mndez
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    COPYNG file for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **/


/*
 BUGS:

 If all the lights are deleted, the lighting in opengl shouldn't be disabled.
 That is the function of the "lighting" radio button. This button doesn't work
 because it calls 'glights.gl_disable_all', but in "libsuperficie/", several
 files call 'slights.gl_enable_all' before drawing something, thus ignoring
 the state of the button.

 There should be one variable in SceneLights with the sole purpose of saving the
 state of this button, and perhaps 'SceneLights::some_active' should check first
 the state of this variable, and if it is FALSE, return FALSE ignoring the state
 of the individual lights.
*/


#include "global.h"
#include "support.h"
#include "superficie.h"
#include "windows.h"
#include "gnome-sup.h"
#include "dialog-scene-properties.h"
#include "lights.h"

#if HAVE_CONFIG_H
#       include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <gnome.h>
#include <gtk/gtk.h>
#include <GL/glut.h>
#include <string.h>

#include <iomanip>
#include <deque>
#include <strstream>

static deque<GtkWidget*> gtklights;
static gboolean up_down_linked = TRUE;
static void properties_init();


void
on_normalize_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
    TheScene.normalize(togglebutton->active);

    TheScene.make_current_gl();
#ifdef GL_VERSION_1_2
    TheScene.normalize() ?  glEnable(GL_RESCALE_NORMAL) : glDisable(GL_RESCALE_NORMAL);
#endif
    //glFlush();

    TheScene.redraw();
}


void
on_show_axis_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
    TheScene.axis(togglebutton->active);
    TheScene.redraw();
}

void
on_vpoint_x_changed(GtkSpinButton *spinbutton, gpointer user_data)
{
    TheScene.viewpoint()[0] = gtk_spin_button_get_value_as_float(spinbutton);
    TheScene.redraw();
}

void
on_vpoint_y_changed(GtkSpinButton *spinbutton, gpointer user_data)
{
    TheScene.viewpoint()[1] = gtk_spin_button_get_value_as_float(spinbutton);
    TheScene.redraw();
}

void
on_vpoint_z_changed(GtkSpinButton *spinbutton, gpointer user_data)
{
    TheScene.viewpoint()[2] = gtk_spin_button_get_value_as_float(spinbutton);
    TheScene.redraw();
}


void
on_fovy_changed(GtkSpinButton *spinbutton, gpointer user_data)
{
    TheScene.fovy(gtk_spin_button_get_value_as_float(spinbutton));
    TheScene.redraw();
}


void
on_ligth_two_side_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
    TheScene.two_side_lighting(togglebutton->active);
    TheScene.make_current_gl();
    glLightModeli( GL_LIGHT_MODEL_TWO_SIDE, togglebutton->active );
    //glFlush();
    TheScene.redraw();
}


void
on_lowm_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
    TheScene.no_light_when_moving(togglebutton->active);
}


void
on_colorpicker1_color_set              (GnomeColorPicker *gnomecolorpicker,
                                        guint            arg1,
                                        guint            arg2,
                                        guint            arg3,
                                        guint            arg4,
                                        gpointer         user_data)
{
    gdouble color[4];

    gnome_color_picker_get_d( gnomecolorpicker,
                              color,
                              color + 1,
                              color + 2,
                              color + 3 );

    TheScene.bg_color(color);

    glClearColor( TheScene.bg_color()[0],
                  TheScene.bg_color()[1],
                  TheScene.bg_color()[2],
                  TheScene.bg_color()[3]);

    ostrstream color_spec;
    color_spec << setfill('0');
    color_spec << hex << "rgb:" <<
        setw(4) << arg1 <<"/" <<
        setw(4) << arg2 << "/" <<
        setw(4) << arg3 << ends;
    gnome_config_set_string("/Superficie/scene/bg_color", color_spec.str());
    color_spec.freeze(0);
    gnome_config_sync();
    TheScene.redraw();
}


void
on_near_changed                        (GtkSpinButton *spinbutton,
                                        gpointer         user_data)
{
    TheScene.near(gtk_spin_button_get_value_as_float(spinbutton));
    TheScene.make_current_gl();
    TheScene.window_reshape();
    //glFlush();
    TheScene.redraw();
}


void
on_far_changed                         (GtkSpinButton *spinbutton,
                                        gpointer         user_data)
{
    TheScene.far(gtk_spin_button_get_value_as_float(spinbutton));
    TheScene.make_current_gl();
    TheScene.window_reshape();
    //glFlush();
    TheScene.redraw();
}

void
on_light_ambient_activate              (GtkEditable     *editable,
                                        gpointer         user_data)
{
    GtkWidget * lights_notebook;
    gint               page_num;
    gfloat              data[4] = {0,0,0,1};

    istrstream entrada(gtk_entry_get_text(GTK_ENTRY(editable)));

    lights_notebook = lookup_widget(Properties, "lights_notebook");
    page_num        = gtk_notebook_get_current_page( GTK_NOTEBOOK(lights_notebook) );

    entrada >> data[0] >> data[1] >> data[2];

    TheScene.make_current_gl();
    slights.update_ambient(page_num, (gfloat*) data);
    //glFlush();

    TheScene.redraw();
}


void
on_light_position_activate             (GtkEditable     *editable,
                                        gpointer         user_data)
{
    GtkWidget * lights_notebook;
    gint               page_num;
    gfloat              data[4] = {0,0,0,0};

    istrstream entrada(gtk_entry_get_text(GTK_ENTRY(editable)));

    lights_notebook = lookup_widget(Properties, "lights_notebook");
    page_num        = gtk_notebook_get_current_page( GTK_NOTEBOOK(lights_notebook) );

    entrada >> data[0] >> data[1] >> data[2];
    slights.update_position(page_num, (gfloat*) data);
    TheScene.redraw();
}


void
on_light_diffuse_activate              (GtkEditable     *editable,
                                        gpointer         user_data)
{
    GtkWidget * lights_notebook;
    gint               page_num;
    gfloat              data[4] = {0,0,0,1};

    istrstream entrada(gtk_entry_get_text(GTK_ENTRY(editable)));

    lights_notebook = lookup_widget(Properties, "lights_notebook");
    page_num        = gtk_notebook_get_current_page( GTK_NOTEBOOK(lights_notebook) );

    entrada >> data[0] >> data[1] >> data[2];

    TheScene.make_current_gl();
    slights.update_diffuse(page_num, (gfloat*) data);
    //glFlush();


    TheScene.redraw();

}


void
on_light_specular_activate             (GtkEditable     *editable,
                                        gpointer         user_data)
{
    GtkWidget * lights_notebook;
    gint               page_num;
    gfloat              data[4] = {0,0,0,1};

    istrstream entrada(gtk_entry_get_text(GTK_ENTRY(editable)));

    lights_notebook = lookup_widget(Properties, "lights_notebook");
    page_num        = gtk_notebook_get_current_page( GTK_NOTEBOOK(lights_notebook) );

    entrada >> data[0] >> data[1] >> data[2];

    TheScene.make_current_gl();
    slights.update_specular(page_num, (gfloat*) data);
    //glFlush();

    TheScene.redraw();
}


void
on_light_active_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    GtkWidget * lights_notebook;
    gint               page_num;

    lights_notebook = lookup_widget(Properties, "lights_notebook");
    page_num        = gtk_notebook_get_current_page( GTK_NOTEBOOK(lights_notebook) );

    TheScene.make_current_gl();
    slights.update_active(page_num, togglebutton->active);
    //glFlush();

    TheScene.redraw();
}

// This function creates the window that contains "lights_data", but doesn't show
// the window. But within the code generated by glade, all the object data is atached
// to the window, so we need to add the window as data object to the table ("lights_data")
// in order to be able to get all the contained widgets and use it later.
// I think this is very ugly. If someone think a better way of doing things, please
// let me know.

GtkWidget*
make_lights_data (gchar *widget_name, gchar *string1, gchar *string2,
                  gint int1, gint int2)
{
    GtkWidget * lights_data;
    GtkWidget * window;

    window = create_window1();
    lights_data = lookup_widget(window, "lights_data");
    gtk_container_remove( GTK_CONTAINER(window), lights_data );
    gtk_object_set_data( GTK_OBJECT(lights_data), "parent_widget", window );

    return lights_data;
}

static void
update_entry(GtkWidget * data_widget, gchar * widget_name, SVector<4> & data, gint page_num)
{
    ostrstream label;
    GtkWidget * widget;

    label << data[0] << " " << data[1] << " " << data[2] << ends;

    widget = (GtkWidget*) gtk_object_get_data (GTK_OBJECT (data_widget),  widget_name);
    gtk_signal_handler_block_by_data(GTK_OBJECT(widget), NULL);
    gtk_entry_set_text( GTK_ENTRY(widget), label.str() );
    gtk_signal_handler_unblock_by_data(GTK_OBJECT(widget), NULL);

    label.freeze(0);

}


void
on_defaults_light_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget * lights_notebook = lookup_widget(Properties, "lights_notebook");
    GtkWidget * gtklabel;

    // Disable all lights
    TheScene.make_current_gl();
    slights.erase_all();
    //glFlush();

    gint i = gtklights.size();

    // remove all data
    while( i -- > 0 ) {
        gtk_notebook_remove_page( GTK_NOTEBOOK(lights_notebook), i );
        gtklights.erase( gtklights.begin() + i );
    }

    slights.make_defaults();

    TheScene.make_current_gl();
    slights.update_all();
    //glFlush();

    properties_init();
    properties_update();


    GtkWidget * widget = lookup_widget(Properties, "remove_light");
    gtk_widget_set_sensitive( GTK_WIDGET(widget), TRUE );

    widget = lookup_widget(Properties, "add_light");
    gtk_widget_set_sensitive( GTK_WIDGET(widget), TRUE );

}


// add a page to the end of the deque (gtklights), and to the end of the notebook

void
on_add_light_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
    gint n = slights.size();

    g_assert( n < MAX_LIGHTS );

    GtkWidget * lights_notebook = lookup_widget(Properties, "lights_notebook");

    ostrstream label;
    label << n + 1 << ends;
    GtkWidget * gtklabel = gtk_label_new( label.str() );

    // add a new page to the deque
    gtklights.push_back( make_lights_data("ligth_data_page", NULL, NULL, 0, 0) );
    // add the label as an object data
    gtk_object_set_data( GTK_OBJECT(gtklights.back()), "label", gtklabel );
    // add a new light to the scene
    slights.push_back();

    gtk_widget_show( gtklights.back() );
    gtk_notebook_append_page( GTK_NOTEBOOK(lights_notebook), gtklights.back(), gtklabel );
    label.freeze(0);

    // parent_widget contains the pointers to the page data
    GtkWidget * parent_widget = (GtkWidget*) gtk_object_get_data (GTK_OBJECT (gtklights.back()), "parent_widget");
    GtkCheckButton * active = (GtkCheckButton*)
        gtk_object_get_data (GTK_OBJECT (parent_widget), "light_active");

    // make gtklights[n] the active page (on_light_active_toggled relies on this)
    gtk_notebook_set_page( GTK_NOTEBOOK(lights_notebook), n );

    {
        // This will emite the "toggled" signal
        gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(active), TRUE );

        TheScene.make_current_gl();
        slights.update_ambient(n);
        slights.update_diffuse(n);
        slights.update_specular(n);
        //glFlush();

    }


    // update add/remove buttons

    if( slights.size() == 1 ) {
        GtkWidget * remove_button = lookup_widget(Properties, "remove_light");
        gtk_widget_set_sensitive( GTK_WIDGET(remove_button), TRUE );
    }
    else if( slights.size() == MAX_LIGHTS ) {
        gtk_widget_set_sensitive( GTK_WIDGET(button), FALSE );
    }

    properties_update();
    TheScene.redraw();
}


void
on_remove_light_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
    g_assert( slights.size() > 0 );

    GtkWidget * lights_notebook = lookup_widget(Properties, "lights_notebook");
    gint page_num = gtk_notebook_get_current_page( GTK_NOTEBOOK(lights_notebook) );
    GtkWidget * parent_widget = (GtkWidget*) gtk_object_get_data (GTK_OBJECT (gtklights[page_num]), "parent_widget");


    gtk_notebook_remove_page( GTK_NOTEBOOK(lights_notebook), page_num );
    gtk_object_destroy( GTK_OBJECT(gtklights[page_num]) );
    gtklights.erase( gtklights.begin() + page_num );


    TheScene.make_current_gl();
    slights.erase( page_num );
    //glFlush();


    if( slights.size() == 0 ) {
        gtk_widget_set_sensitive( GTK_WIDGET(button), FALSE );
    }
    else if( slights.size() == MAX_LIGHTS - 1 ) {
        GtkWidget * add_button = lookup_widget(Properties, "add_light");
        gtk_widget_set_sensitive( GTK_WIDGET(add_button), TRUE );
    }

    properties_update();
    TheScene.redraw();
}


void
on_lighting_on_toggled                 (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    TheScene.make_current_gl();
    togglebutton->active ? slights.gl_enable_all() : slights.gl_disable_all();
    //glFlush();

    TheScene.redraw();
}

void
on_feedbufer_size_changed              (GtkEditable     *editable,
                                        gpointer         user_data)
{
    //TODO: This should adjust automatically.
    gulong size;
    istrstream entrada(gtk_entry_get_text(GTK_ENTRY(editable)));
    entrada >> size;
    TheScene.feedBufferSize(size);
}


void
on_pstoedit_activate                   (GtkEditable     *editable,
                                        gpointer         user_data)
{
    g_free(Pstoedit);
    Pstoedit = gtk_editable_get_chars(editable, 0, -1);
    gnome_config_set_string("/Superficie/scene/pstoedit", Pstoedit);
    gnome_config_sync();
}


void
on_ps_viewer_activate                  (GtkEditable     *editable,
                                        gpointer         user_data)
{
    g_free(Psviewer);
    Psviewer = gtk_editable_get_chars(editable, 0, -1);
    gnome_config_set_string("/Superficie/scene/psviewer", Psviewer);
    gnome_config_sync();
}


void
on_xfig_activate                       (GtkEditable     *editable,
                                        gpointer         user_data)
{
    g_free(Xfig);
    Xfig = gtk_editable_get_chars(editable, 0, -1);
    gnome_config_set_string("/Superficie/scene/xfig", Xfig);
    gnome_config_sync();
}


void
on_sketch_activate                     (GtkEditable     *editable,
                                        gpointer         user_data)
{
    g_free(Sketch);
    Sketch = gtk_editable_get_chars(editable, 0, -1);
    gnome_config_set_string("/Superficie/scene/sketch", Sketch);
    gnome_config_sync();
}


// cases 0 .. 4 are from the "show as" menu (object properties window)
// cases 5 .. 9 are from the "when moving show as" menu (scene properties window)

void
cb_show_as_menu_select(GtkWidget *item, gpointer data)
{
    BaseObject* ob = TheScene.selected_object();

    switch ( GPOINTER_TO_INT(data) )
    {
        // object properties menu options
    case 0:
        ob->polygon_mode(POLY_FILL);
        break;

    case 1:
        ob->polygon_mode(POLY_HLINE);
        break;

    case 2:
        ob->polygon_mode(POLY_VLINE);
        break;

    case 3:
        ob->polygon_mode(POLY_LINE);
        break;

    case 4:
        ob->polygon_mode(POLY_POINT);
        break;

        // scene properties menu options

    case 5:
        // no change
        Object::polygon_mode_down(POLY_NONE);
        break;

    case 6:
        Object::polygon_mode_down(POLY_FILL);
        break;

    case 7:
        Object::polygon_mode_down(POLY_LINE);
        break;

    case 8:
        Object::polygon_mode_down(POLY_POINT);
        break;

    }

    TheScene.redraw();
}


void
properties_update()
{
    GtkWidget* widget;
    GtkWidget* parent_widget;

    if( !Properties)
        return;

    //  Scene 

    widget = lookup_widget(Properties, "normalize");
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (widget), TheScene.normalize());

    widget = lookup_widget(Properties, "show_axis");
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (widget), TheScene.axis());

    widget = lookup_widget(Properties, "fovy");
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), TheScene.fovy());

    widget = lookup_widget(Properties, "vpoint_x");
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), TheScene.viewpoint()[0]);

    widget = lookup_widget(Properties, "vpoint_y");
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), TheScene.viewpoint()[1]);

    widget = lookup_widget(Properties, "vpoint_z");
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), TheScene.viewpoint()[2]);

    widget = lookup_widget(Properties, "near");
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), TheScene.near());

    widget = lookup_widget(Properties, "far");
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), TheScene.far());

    widget = lookup_widget(Properties, "colorpicker1");
    gnome_color_picker_set_d( GNOME_COLOR_PICKER(widget),
                              TheScene.bg_color()[0],
                              TheScene.bg_color()[1],
                              TheScene.bg_color()[2],
                              TheScene.bg_color()[3] );

    widget = lookup_widget(Properties, "pstoedit");
    gtk_entry_set_text( GTK_ENTRY(widget), Pstoedit );

    widget = lookup_widget(Properties, "ps_viewer");
    gtk_entry_set_text( GTK_ENTRY(widget), Psviewer );

    widget = lookup_widget(Properties, "xfig");
    gtk_entry_set_text( GTK_ENTRY(widget), Xfig );

    widget = lookup_widget(Properties, "sketch");
    gtk_entry_set_text( GTK_ENTRY(widget), Sketch );

    //  lights 

    widget = lookup_widget(Properties, "lighting_on");
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (widget), slights.some_active());

    for(gint i = 0; i < slights.size(); i++ ) {
        parent_widget = (GtkWidget*) gtk_object_get_data (GTK_OBJECT (gtklights[i]), "parent_widget");

        update_entry( parent_widget, "light_ambient",  slights[i].ambient(),  i );
        update_entry( parent_widget, "light_position", slights[i].position(), i );
        update_entry( parent_widget, "light_diffuse",  slights[i].diffuse(),  i );
        update_entry( parent_widget, "light_specular", slights[i].specular(), i );

        widget = (GtkWidget*) gtk_object_get_data( GTK_OBJECT(parent_widget), "light_active" );
        gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (widget),
                                    slights[i].active());

        widget = (GtkWidget*) gtk_object_get_data ( GTK_OBJECT(gtklights[i]), "label" );
        ostrstream label;
        label << i + 1 << ends;
        gtk_label_set_text( GTK_LABEL(widget), label.str() );
        label.freeze(0);
    }
}

GtkWidget*
make_menu_item(gchar *name, GtkSignalFunc callback, gpointer data)
{
    GtkWidget *item;
    
    item = gtk_menu_item_new_with_label (name);
    gtk_signal_connect (GTK_OBJECT (item), "activate",
                        callback, data);
    gtk_widget_show (item);
    
    return item;
}



void
properties_init()
{

    GtkWidget * lights_notebook = lookup_widget(Properties, "lights_notebook");
    GtkWidget * gtklabel;
    GtkWidget * page;

    // This page is added by glade, and is not needed.
    gtk_notebook_remove_page( GTK_NOTEBOOK(lights_notebook), 0 );

    for(gint i = 0; i < slights.size(); i++ ) {

        ostrstream label;
        label << i + 1 << ends;
        gtklabel = gtk_label_new("");

        gtklights.push_back( make_lights_data("ligth_data_page", NULL, NULL, 0, 0) );
        gtk_object_set_data( GTK_OBJECT(gtklights[i]), "label", gtklabel);
        gtk_widget_show(gtklights[i]);
        gtk_notebook_append_page( GTK_NOTEBOOK(lights_notebook), gtklights[i], gtklabel );
        label.freeze(0);
    }



    // Button down menu

    GtkWidget *widget, *menu, *item;

    gchar* menu_labels[] = {
        N_("No change"), N_("Normal"), N_("WireFrame"), N_("Points")
    };

    menu = gtk_menu_new();

    for(gint i=0; i < 4; i++) {
        item = make_menu_item(menu_labels[i],
                              GTK_SIGNAL_FUNC(cb_show_as_menu_select),
                              GINT_TO_POINTER(i+5)); // 5 to 8
        gtk_menu_append(GTK_MENU (menu), item);
    }

    widget = lookup_widget(Properties, "button_down_menu");
    gtk_option_menu_set_menu (GTK_OPTION_MENU (widget), menu);

}



void
scene_properties_cb (GtkWidget* w, gpointer data)
{

    if( !Properties )
    {
#ifdef LIBGLADE
        Properties = glade_xml_get_widget(xml, "Properties");
#else
        Properties = create_Properties();
#endif
        properties_init();
        properties_update();
    }
    
    if( !GTK_WIDGET_VISIBLE (Properties) )
        gtk_widget_show(Properties);

}

