/* main-window.cc
** Copyright (C) 1999 Juan Pablo Romero <juan_pablo_22@yahoo.com>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#if HAVE_CONFIG_H
#       include <config.h>
#endif

#include <gnome.h>

#include "global.h"
#include "gnome-sup.h"
#include "superficie.h"
#include "dialog-object-properties.h"
#include "dialog-scene-properties.h"
#include "dialog-load.h"
#include "dialog-save-as.h"
#include "svector.h"
#include "support.h"

#include "pixmaps/reset-big.xpm"
#include "pixmaps/eye.xpm"
#include "pixmaps/frame-big.xpm"
#include "pixmaps/mesh-color.xpm"

#include <strstream>
#include <libgnome/gnome-history.h>

const gchar *authors[] = {
    "Juan Pablo Romero Mndez",
    NULL
};



void
show_help (GtkWidget* w, gpointer data)
{

}

void
exit_aplication (GtkWidget* w, gpointer data)
{
    gtk_main_quit();
}

void
menu_close_option (GtkWidget* widget, gpointer data)
{
    GtkWidget* tmp = (GtkWidget *) data;
    gtk_widget_destroy( tmp );
}

void
item_activated (GtkWidget *widget, gpointer data)
{
    g_message("%s activated", (char *) data);
}


/*
 *      destroy_window: actually destroy the window
 */

void
destroy_window (GtkWidget* window, gpointer *data)
{

    /*   when this function ends, gtk destroys w
     *   (the window). This causes scene_destroy
     *   to be called.
     */

    gtk_main_quit();
}



/*  close_window: is called in response to a
 *  'delete' event.
 */

gint
close_window (GtkWidget* w, gpointer data)
{
    /*      when we have something to save this
     *      function will useful.
     *      By now, the only thing it does is to call
     *      destroy_window (by returning FALSE).
     */

    return FALSE;
}


void
show_about (GtkWidget* w, gpointer data)
{
    gtk_widget_show( gnome_about_new(_(PACKAGE), VERSION,
                                     "(C) 1998,1999,2000 Juan Pablo Romero",
                                     authors,
                                     _("Web: http://superficie.sourceforge.net"),
                                     "superficie-logo.png"));
}


void
delete_selected_object_cb(GtkWidget* w, gpointer data)
{
    app_delete_node(TheScene.selected_node());
}


void
stop_animation_cb(GtkWidget* w, gpointer data)
{
    TheScene.remove_idle();
    TheScene.idle_all(FALSE);
    g_timer_start(TheScene.timer());
    TheScene.redraw();
}

void
reset_viewpoint_cb(GtkWidget* w, gpointer data)
{
    TheScene.update_viewpoint();
    TheScene.redraw();
}

void
reset_position_cb(GtkWidget* w, gpointer data)
{
    BaseObject* ob = TheScene.selected_object();

    ob->offset( & SVector<3>() );
    gfloat q[] = {0,0,0,1};
    ob->curquat(q);

    TheScene.redraw();
}

void
show_frame_cb(GtkWidget* w, gpointer data)
{
    BaseObject* ob = TheScene.selected_object();

    ob->show_bbox((ob->show_bbox() + 1) % 2);
    tree_update();
    TheScene.redraw();
}

void
show_mesh_cb(GtkWidget* w, gpointer data)
{
    BaseObject* ob = TheScene.selected_object();

    ob->show_mesh((ob->show_mesh() + 1) % 2);
    /* This obscures the frame, so better update de buttons */
    if( ob->show_mesh() )
        ob->smooth_edges(FALSE);

    tree_update();
    TheScene.redraw();
}



gint
button_press (GtkWidget* widget, GdkEventButton* event)
{
    TheScene.update_state(event);
    TheScene._event_button = event->button;
    TheScene.button_press();
}


gint
motion_notify(GtkWidget *glarea, GdkEventMotion *event)
{
    // Creo que no es lo mas eficiente, pero ni modo...
    gint i;
    gfloat vp[3];
    GtkWidget* widget;
    
    TheScene.update_state(event);

    vp[2] = TheScene.viewpoint()[2];

    TheScene.motion_notify();

    if( vp[2] != TheScene.viewpoint()[2] && Properties)
    {
        widget = lookup_widget(Properties, "vpoint_z");
        gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), TheScene.viewpoint()[2]);
    }
}



gint
button_release (GtkWidget* widget, GdkEventButton* event)
{
    BaseObject* ob = TheScene.selected_object();

    TheScene.update_state(event);
    TheScene._event_button = event->button;
    TheScene.button_release();
}



gint
key_press (GtkWidget* w, GdkEventKey* event, gpointer data)
{
    BaseObject* ob;

    switch (event->keyval)
    {
    case GDK_Insert:
        app_insert_object(NULL, NULL);
        return TRUE;
        break;

    case GDK_Escape:
        /*if( loading )
         t_message.abort_loading = TRUE;
         return TRUE;
         */
        break;

    case GDK_Delete:
        app_delete_node(TheScene.selected_node());
        return TRUE;
        break;

    case GDK_t:
        ob = TheScene.selected_object();
        ob->transparent( (ob->transparent() + 1) % 2 );
        tree_update();
        TheScene.redraw();
        return TRUE;
        break;

    case GDK_c:
        TheScene.center().print(cout);
        break;

    };
    return FALSE;
}


gint
init(GtkWidget *widget)
{
    if (gtk_gl_area_make_current(GTK_GL_AREA(widget))) {
        TheScene.realize();
    }
    return TRUE;
}


gint
draw(GtkWidget *widget, GdkEventExpose *event)
{
  if (event->count > 0)
    return TRUE;

  TheScene.redraw();

  return TRUE;
}


gint
reshape(GtkWidget *widget, GdkEventConfigure *event)
{
    if (gtk_gl_area_make_current(GTK_GL_AREA(widget))) {
        TheScene.update_state(event);
        TheScene.window_reshape();
    }
    return TRUE;
}


void
run_viewer(GtkWidget* w, gpointer program)
{
    gchar* argv[2];
    gchar* pwd;
    gchar * ps_tmp = NULL;
    gchar * fig_tmp = NULL;

    ps_tmp = tempnam(NULL, "sup");

    TheScene.print(ps_tmp);

    ostrstream command;

    if(GPOINTER_TO_INT(program)) {
        argv[0] = Xfig;
        fig_tmp = tempnam(NULL, "supex");
        command << Pstoedit << " -f fig " << ps_tmp << " " << fig_tmp << ends;
        system(command.str());
        argv[1] = fig_tmp;
        command.freeze(0);
    }
    else {
        argv[0] = Psviewer;
        argv[1] = ps_tmp;
    }

    gnome_execute_async(g_get_current_dir(), 2,  argv);

    free(ps_tmp);
    free(fig_tmp);
}



void
added_item_callback(GtkWidget* w, gchar* file)
{
    update_window_from_file_async(file);
}


void
fill_menu_array( gpointer gpentry, gpointer data )
{
    GnomeHistoryEntry real_entry = (GnomeHistoryEntry) gpentry;
    gchar buf[200];
    GnomeUIInfo entry[2];
    int pos = 0;
    string type(((GnomeHistoryEntry) real_entry)->filetype);
    string basename(g_basename(real_entry->filename));

    if( string("application/x-superficie") == type ) {
        entry[0].type = GNOME_APP_UI_ITEM;
        entry[0].label = basename.c_str();
        entry[0].hint = NULL;
        entry[0].moreinfo = added_item_callback;
        entry[0].user_data = real_entry->filename;
        entry[0].unused_data = NULL;
        entry[0].pixmap_type = GNOME_APP_PIXMAP_NONE;
        entry[0].pixmap_info = NULL;
        entry[0].accelerator_key = 0;
        entry[1].type = GNOME_APP_UI_ENDOFINFO;
        
        gnome_app_insert_menus (GNOME_APP (TheWindow), "File/Previous/", entry);
    }
    return;
}


static gint
compare_names(gconstpointer a, gconstpointer b)
{
    gint ret;
    
    GnomeHistoryEntry entry_a = ((GnomeHistoryEntry) a);
    GnomeHistoryEntry entry_b = ((GnomeHistoryEntry) b);
    
    ret = strcmp(entry_a->filename, entry_b->filename);
    
    return ret;
}


gint
progress_timeout(gpointer data)
{
    gdouble  progress = 0.0;
    GList* mru_files;
    GList* file_found;

    gdk_threads_enter();

    if( t_message.abort_loading ) {
        gnome_appbar_set_status(GNOME_APPBAR(GNOME_APP(TheWindow)->statusbar), _("loading canceled!"));
        gnome_appbar_set_progress(GNOME_APPBAR(GNOME_APP(TheWindow)->statusbar), 0.0);
        t_message.abort_loading = FALSE;
        gdk_threads_leave();
        return FALSE;
    }

    progress = (gdouble) t_message.triangle_procesed / (gdouble) t_message.triangle_total;
    gnome_appbar_set_progress(GNOME_APPBAR(GNOME_APP(TheWindow)->statusbar), progress);

    if( t_message.set_loading_message ) {
        gnome_appbar_set_status(GNOME_APPBAR(GNOME_APP(TheWindow)->statusbar), _("loading..."));
        t_message.set_loading_message = FALSE;
    }

    if( t_message.generating_normals ) {
        gnome_appbar_set_status(GNOME_APPBAR(GNOME_APP(TheWindow)->statusbar), _("calculating normals..."));
        t_message.generating_normals = FALSE;
    }

    if(!loading) {
        // All files finished loading
        TheScene.redraw();
        gnome_appbar_set_progress(GNOME_APPBAR(GNOME_APP(TheWindow)->statusbar), 1.0);
        gnome_appbar_set_status(GNOME_APPBAR(GNOME_APP(TheWindow)->statusbar), _("done!"));

        BaseObject* ob = TheScene.selected_object();

        ostrstream window_title;
        window_title << PACKAGE << " " << VERSION << ends;

        gtk_window_set_title(GTK_WINDOW (TheWindow), window_title.str());
        window_title.freeze(0);

        // if loading of the file was succesfull
        if( t_message.insert_menu_item ) {
            GnomeHistoryEntry entry = g_new(struct _GnomeHistoryEntry, 1);
            entry->filename = t_message.insert_menu_item;
            entry->filetype = "application/x-superficie";

            mru_files = gnome_history_get_recently_used();
            file_found = g_list_find_custom(mru_files, entry, compare_names);

            // if file isn't found and name doesn't start with "/tmp" and
            // name is an absolute path
            if( !file_found &&
                string(entry->filename).substr(0,4) != string("/tmp") &&
                string(entry->filename) != string("") &&
                g_path_is_absolute (entry->filename)
              )
            {
                gnome_history_recently_used(entry->filename, "application/x-superficie", "superficie", _("carga archivo"));
                fill_menu_array(entry, NULL);
            }
            t_message.insert_menu_item = NULL;
        }
        rebuild_tree();
        app_select_node( TheScene.selected_node() );

        return FALSE;
    }
    gdk_threads_leave();
    return TRUE;
}


GtkWidget* make_glarea(GtkWidget* parent)
{
    GtkWidget* glarea;

    glarea = GTK_WIDGET(gtk_gl_area_new_vargs(NULL, /* no sharing */
                                              GDK_GL_RGBA,
                                              GDK_GL_DOUBLEBUFFER,
                                              GDK_GL_DEPTH_SIZE,1,
                                              GDK_GL_NONE));
    gtk_widget_set_events(glarea,
                          GDK_EXPOSURE_MASK|
                          GDK_BUTTON_PRESS_MASK|
                          GDK_BUTTON_RELEASE_MASK|
                          GDK_POINTER_MOTION_MASK|
                          GDK_POINTER_MOTION_HINT_MASK);
    
    gtk_signal_connect( GTK_OBJECT(glarea), "expose_event",
                        GTK_SIGNAL_FUNC(draw), NULL );
    gtk_signal_connect( GTK_OBJECT(glarea), "configure_event",
                        GTK_SIGNAL_FUNC(reshape), NULL );
    gtk_signal_connect( GTK_OBJECT(glarea), "realize",
                        GTK_SIGNAL_FUNC(init), NULL );
    
    gtk_signal_connect( GTK_OBJECT(glarea), "button_press_event",
                        GTK_SIGNAL_FUNC (button_press), NULL );
    gtk_signal_connect( GTK_OBJECT(glarea), "button_release_event",
                        GTK_SIGNAL_FUNC (button_release), NULL );
    gtk_signal_connect( GTK_OBJECT(glarea), "motion_notify_event",
                        GTK_SIGNAL_FUNC(motion_notify), NULL );
    
    GTK_WIDGET_SET_FLAGS(glarea, GTK_CAN_FOCUS);
    gtk_widget_grab_focus(GTK_WIDGET(glarea));
    
    gtk_widget_set_usize(GTK_WIDGET(glarea), 300, 300);
    gnome_app_set_contents(GNOME_APP(parent), glarea);

    return glarea;
}



GtkWidget*
create_main (gchar* title)
{
    
    GtkWidget*   window;
    GnomeAppBar* bar;

    window = gnome_app_new("Superficie", title);
    gtk_window_set_default_size(GTK_WINDOW(window), 300,300);

    gtk_signal_connect(GTK_OBJECT(window), "delete_event",
                       GTK_SIGNAL_FUNC(close_window),
                       NULL);
    gtk_signal_connect(GTK_OBJECT (window), "destroy",
                       GTK_SIGNAL_FUNC (destroy_window),
                       NULL);

    gtk_signal_connect(GTK_OBJECT (window), "key_press_event",
                       GTK_SIGNAL_FUNC (key_press), NULL);

    GnomeUIInfo program_menu[] = {
        GNOMEUIINFO_MENU_CLOSE_ITEM(menu_close_option, window),
        GNOMEUIINFO_SEPARATOR,
        GNOMEUIINFO_MENU_EXIT_ITEM(exit_aplication, NULL),
        GNOMEUIINFO_END
    };
    

    GnomeUIInfo ops_toolbar[] = {
        { GNOME_APP_UI_ITEM, N_("stop"), N_("stop the animation"), stop_animation_cb, NULL, NULL,
        GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_STOP, 0, (GdkModifierType)0, NULL },

        { GNOME_APP_UI_ITEM, N_("viewpoint"), N_("reset viewpoint"), reset_viewpoint_cb, NULL, NULL,
        GNOME_APP_PIXMAP_DATA, eye_xpm, 0, (GdkModifierType)0, NULL },

        { GNOME_APP_UI_ITEM, N_("reset"), N_("reset object position"), reset_position_cb, NULL, NULL,
        GNOME_APP_PIXMAP_DATA, reset_big_xpm, 0, (GdkModifierType)0, NULL },

        { GNOME_APP_UI_ITEM, N_("frame"), N_("toggle frame"), show_frame_cb, NULL, NULL,
        GNOME_APP_PIXMAP_DATA, frame_big_xpm, 0, (GdkModifierType)0, NULL },

        { GNOME_APP_UI_ITEM, N_("mesh"), N_("toggle mesh"), show_mesh_cb, NULL, NULL,
        GNOME_APP_PIXMAP_DATA, mesh_color_xpm, 0, (GdkModifierType)0, NULL },

        GNOMEUIINFO_END
    };


    GnomeUIInfo open_previous_menu[] = {
        { GNOME_APP_UI_ENDOFINFO }
    };

    GnomeUIInfo file_menu [] = {
        GNOMEUIINFO_MENU_OPEN_ITEM(load_cb, NULL),
        GNOMEUIINFO_MENU_SAVE_AS_ITEM(save_as_cb, NULL),

        {GNOME_APP_UI_ITEM, N_("View postscript"), N_("Run the PS viewer"),
        run_viewer, GINT_TO_POINTER(0), NULL, GNOME_APP_PIXMAP_NONE, NULL,
        0, (GdkModifierType) 0, NULL},

        {GNOME_APP_UI_ITEM, N_("View xfig"), N_("Run xfig"),
        run_viewer, GINT_TO_POINTER(1), NULL, GNOME_APP_PIXMAP_NONE, NULL,
        0,  (GdkModifierType) 0, NULL},

        GNOMEUIINFO_SEPARATOR,

        { GNOME_APP_UI_SUBTREE, N_("_Previous"), NULL,
        open_previous_menu, NULL, NULL, GNOME_APP_PIXMAP_NONE, NULL,
        0,  (GdkModifierType) 0, NULL },

        GNOMEUIINFO_SEPARATOR,

        GNOMEUIINFO_MENU_EXIT_ITEM(exit_aplication, NULL),

        GNOMEUIINFO_END
    };
    
    GnomeUIInfo edit_menu [] = {
        GNOMEUIINFO_MENU_CLEAR_ITEM(delete_selected_object_cb, NULL),
        GNOMEUIINFO_END
    };
    
    GnomeUIInfo windows_menu[] = {
        {GNOME_APP_UI_ITEM, N_("_Object properties..."), N_("Open the Object manager window"),
        object_properties_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP,
        'O', GDK_CONTROL_MASK, NULL},

        {GNOME_APP_UI_ITEM, N_("_Scene properties..."), N_("Open the Scene properties  window"),
        scene_properties_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP,
        'S', GDK_CONTROL_MASK, NULL},

        GNOMEUIINFO_END
    };
    
    
    GnomeUIInfo help_menu[] = {
        GNOMEUIINFO_HELP ("superficie"),
        GNOMEUIINFO_MENU_ABOUT_ITEM(show_about,NULL),
        GNOMEUIINFO_END
    };
    
    GnomeUIInfo superficie_menu [] = {
        GNOMEUIINFO_SUBTREE (N_("_File"), file_menu),
        GNOMEUIINFO_SUBTREE (N_("_Windows"), windows_menu),
        GNOMEUIINFO_SUBTREE (N_("_Help"), help_menu),
        GNOMEUIINFO_END
    };
    
    GnomeUIInfo helper_toolbar[] = {
        
        { GNOME_APP_UI_ITEM, N_("Load"), N_("Load an existing object"),
        load_cb, NULL, NULL,
        GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_OPEN, 0, (GdkModifierType) 0, NULL },
        
        { GNOME_APP_UI_ITEM, N_("Save"), N_("Save the current file"),
        save_as_cb, NULL, NULL,
        GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_SAVE, 0,  (GdkModifierType) 0, NULL },

        GNOMEUIINFO_SEPARATOR,
        
        { GNOME_APP_UI_ITEM, N_("Scene"), N_("Change scene properties"),
        scene_properties_cb, NULL, NULL,
        GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_PROPERTIES, 0,  (GdkModifierType) 0, NULL },

        { GNOME_APP_UI_ITEM, N_("Objects"), N_("Manipulate objects"),
        object_properties_cb, NULL, NULL,
        GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_INDEX, 0,  (GdkModifierType) 0, NULL },

        GNOMEUIINFO_END
    };
    
    gnome_app_create_menus  ( GNOME_APP(window), superficie_menu );
    gnome_app_create_toolbar( GNOME_APP(window), helper_toolbar);

    GtkWidget * toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
    //GtkWidget * toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_TEXT);
    gnome_app_fill_toolbar(GTK_TOOLBAR (toolbar), ops_toolbar, NULL);
    gtk_widget_show(toolbar);

    gnome_app_add_docked(GNOME_APP(window), toolbar, "ops", GNOME_DOCK_ITEM_BEH_NORMAL,
                         GNOME_DOCK_RIGHT, 1, 0, 0);

    bar = GNOME_APPBAR(gnome_appbar_new(TRUE, TRUE, GNOME_PREFERENCES_USER));
    gnome_app_set_statusbar( GNOME_APP(window), GTK_WIDGET(bar));
    gnome_app_install_appbar_menu_hints(GNOME_APPBAR(bar), superficie_menu);

    /*
     *     You should always delete gtk_gl_area widgets before exit or else
     *     GLX contexts are left undeleted, this may cause problems (=core dump)
     *     in some systems.
     *     Destroy method of objects is not automatically called on exit.
     *     You need to manually enable this feature. Do gtk_quit_add_destroy()
     *     for all your top level windows unless you are certain that they get
     *     destroy signal by other means.
     */
    gtk_quit_add_destroy(1, GTK_OBJECT(window));

    return window;
}
