/*
 *  superficie.cc
 *  
 *  Copyright (C) 1998, 1999  by Juan Pablo Romero Mndez
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  COPYNG file for more details.
 */

#include "global.h"
#include "app.h"
#include "superficie.h"
#include "main-window.h"
#include "support.h"
#include "gnome-sup.h"
#include "windows.h"

#ifdef PTH
#include <pth.h>
#else
#include <pthread.h>
#endif

#include <math.h>
#include <string.h>
#include <stdio.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#include <unistd.h>
#include <gtkgl/gtkglarea.h>
#include <gnome.h>
#include <libgnome/gnome-history.h>
#include <gdk_imlib.h>

#ifdef LIBGLADE
#include <glade/glade.h>
#endif

#include <strstream>

#if HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>


// ---------------- Global Objects  ---------------------- //

/* Esta funcin es el el primer argumento de 'gtk_idle_add';
 * si la definimos como funcin miembro de GtkScene entonces
 * el compilador hace una advertencia cada vez que se topa con
 * la definicin de la clase.
 * Por eso opt por no definirla como funcin miembro. La otra
 * opcin es crear un archivo (p.ej. 'gtk-sup.cc') que nicamente
 * contenga la definicin de la funcin, pero tendramos una biblioteca
 * mas que enlazar, lo cual sera bastante molesto.
 * Se aceptan sugerencias.
 */

/* This function is the first argument to 'gtk_idle_add';
 * if we define it as a member function of GtkScene, then
 * the compiler shows a warning each time it reads the
 * class definition.
 * That's why I decided not to make it a member function.
 * The alternative is to make a file that only contains the
 * definition of this function, but we would have another object
 * file to link with. I accept sugestions.
 */

gint _gtk_private_redraw(GtkScene* scene) { /*FOLD00*/
    scene->redraw();

}


GnomeScene  TheScene;
GtkWidget * TheWindow = NULL;
GtkWidget * TheTreeW = NULL;
GtkWidget * Properties = NULL;
GtkWidget * info = NULL;

#ifdef LIBGLADE
GladeXML  * xml;
#endif

//paths to helper programs

gchar * Pstoedit = NULL;
gchar * Psviewer = NULL;
gchar * Xfig = NULL;
gchar * Sketch = NULL;


// ........................................................ //
gint pb_timer = 0;
gint loading = 0;


void *
load_object_from_file(void* file) /*FOLD00*/
{
    gboolean read;

    read = TheScene.read((gchar*) file, NULL, 0);
    t_message.set_loading_message = TRUE;
    if (read)
        t_message.insert_menu_item = g_strdup( (char*) file );
    loading--;
    pthread_exit(NULL);
    return NULL;
}



void
update_window_from_file_async(gchar* file) /*FOLD00*/
{
    pthread_t thread;
    GtkWidget *pb;

    if( !loading ) {
        gnome_appbar_set_progress(GNOME_APPBAR(GNOME_APP(TheWindow)->statusbar), 0);
        /* Para que la barra de progreso se vaya a cero */
        t_message.triangle_procesed = 0;
        t_message.triangle_total = 1;
        pb_timer = gtk_timeout_add(100, progress_timeout, NULL);
    }
    loading++;

#ifdef PTH
    pth_spawn(PTH_ATTR_DEFAULT, load_object_from_file, (void*) file);
#else
    pthread_create (&thread, NULL, load_object_from_file, (void*) file);
#endif

}


gint
load_object_from_stream(FILE* fp) /*FOLD00*/
{
    gint read, c;
    gchar * bp;
    size_t size;
    FILE * stream;

    stream = open_memstream (&bp, &size);
    while( (c = getc(fp)) >= 0)
        fprintf(stream, "%c", c);

    fclose(stream);

    read = TheScene.read(NULL, bp, size);
    return read;
}


void
update_window_from_stream(FILE* fp, char* name) /*FOLD00*/
{
    load_object_from_stream(fp);
}



void
load_defaults(gchar * bg_color_op)
{
    gboolean    is_def;
    gchar * color_spec;
    GdkColor gdk_color;
    gdouble   color[4];

    if( bg_color_op ) {
        color_spec = g_strdup(bg_color_op);
    }
    else {
        color_spec =
            gnome_config_get_string_with_default("/Superficie/scene/bg_color=#ffffff", &is_def);
    }

    if( !gdk_color_parse (color_spec, &gdk_color) ) {
        g_warning(_("invalid color spec: %s" ), color_spec);
        _for_each(i, 4, color[i] = 0;);
    }
    else {
        color[0] = gdk_color.red   / (gdouble) 65535;
        color[1] = gdk_color.green / (gdouble) 65535;
        color[2] = gdk_color.blue  / (gdouble) 65535;
        color[3] = 0;
    }
    TheScene.bg_color(color);
    g_free(color_spec);

    Pstoedit =
        gnome_config_get_string_with_default("/Superficie/scene/pstoedit=pstoedit", &is_def);

    Psviewer =
        gnome_config_get_string_with_default("/Superficie/scene/psviewer=gv", &is_def);

    Xfig =
        gnome_config_get_string_with_default("/Superficie/scene/xfig=xfig", &is_def);

    Sketch =
        gnome_config_get_string_with_default("/Superficie/scene/sketch=sketch", &is_def);
}


int
main(int argc, char ** argv) /*FOLD00*/
{
#ifdef PTH
    pth_init();
#endif

    bindtextdomain(PACKAGE, GNOMELOCALEDIR);
    textdomain(PACKAGE);

    gchar ** startup_files = NULL;
    gchar * bg_color = NULL;
    gboolean read_from_stdin = false;
    poptContext ctx;

    struct poptOption prog_options[] = {
        {"bg", '\0', POPT_ARG_STRING, &bg_color, 0, N_("background color"), N_("COLOR")},
        {"stdin", '\0', POPT_ARG_NONE, &read_from_stdin, 0, N_("read data from stdin"), NULL},
        POPT_AUTOHELP
        {NULL, '\0', 0, NULL, 0, NULL, NULL}
    };


    if( gdk_gl_query() == FALSE ) {
        g_error(_("OpenGL not supported"));
        return (0);
    }

    gnome_init_with_popt_table (PACKAGE, VERSION, argc, argv, prog_options, 0, &ctx);
    gdk_imlib_init();

#ifdef LIBGLADE
    glade_gnome_init();
    xml = glade_xml_new("superficie.glade", NULL);
    glade_xml_signal_autoconnect(xml);
#endif

    load_defaults(bg_color);

    /* Creates an empty window */
    ostrstream window_title;
    window_title << PACKAGE << " " << VERSION << ends;
    TheWindow = create_main(window_title.str());
    window_title.freeze(0);
    TheScene.add_glarea(make_glarea(TheWindow));
    gtk_widget_show(GTK_WIDGET(TheWindow));

    /* Fill the m.r.u menu */
    GList * mru_files = gnome_history_get_recently_used();
    g_list_foreach(mru_files, fill_menu_array, NULL);

    startup_files = poptGetArgs (ctx);

    if ( startup_files ) {
        for ( gint i = 0; startup_files[i]; i++ ) {
            update_window_from_file_async( startup_files[i] );
        }
    }
    else
        if( read_from_stdin )
            update_window_from_stream(stdin, "stdin");

    poptFreeContext(ctx);
    g_thread_init(NULL);

    gtk_main();
    return 0;
}


