/***************************************************************************
                          base_object.cc  -  description
                             -------------------
    begin                : Fri Dec 10 1999
    copyright            : (C) 1999 by Juan Pablo Romero Mndez
    email                : juan_pablo_22@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "base_object.h"

#include "read_data.h"
#include "util.h"
#include "trackball.h"

#include <math.h>
#include <string.h>
#include <GL/gl.h>


BaseObject::BaseObject()
{
    _gnode = NULL;
    _gl_load_name = -1;
    _polygon_mode = POLY_FILL;
    _normals  = FALSE;
    _mono     = TRUE;
    _smooth   = FALSE;
    _idle     = FALSE;
    _motion   = FALSE;
    _transparent  = FALSE;
    _invisible    = FALSE;
    _show_mesh    = FALSE;
    _mesh_smooth  = TRUE;
    _smooth_lines = FALSE;
    _smooth_edges = FALSE;

    _frame_line_width = 1.0;
    _lines_width  = 1.0;
    _points_width = 1.3;

    _transparent_array = patron[0];
    gfloat tmp_mono_color[] = {1, 1, 1, 1};
    _mono_color = tmp_mono_color;

    _frame_color  = SVector<4>(0, 0, 0, 1);
    _normal_color = SVector<4>(0, 0, 0, 1);
    _mesh_color   = SVector<4>(0, 0, 0, 1);

    _selected = FALSE;

    _last_motion = 0;
    _previous_last_motion = 0;
    _show_bbox = TRUE;
}

void
BaseObject::initialize()
{
    calc_center();
    update_bbox();
}



void
BaseObject::draw_bbox()
{
    {   // disable lighting
        glDisableClientState(GL_COLOR_ARRAY);
        glDisable(GL_COLOR_MATERIAL);
        slights.gl_disable_all();
        glDisableClientState(GL_NORMAL_ARRAY);
        // set color
        glColor3fv( & _frame_color );
    }
    glLineWidth (lines_width ());

    if( smooth_lines() ) {
        glEnable (GL_LINE_SMOOTH);
        glEnable (GL_BLEND);
        glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    }


    glBegin(GL_LINE_LOOP);
         glVertex3f(_bbox[1], _bbox[3], _bbox[4] );
         glVertex3f(_bbox[0], _bbox[3], _bbox[4] );
         glVertex3f(_bbox[0], _bbox[2], _bbox[4] );
         glVertex3f(_bbox[1], _bbox[2], _bbox[4] );
    glEnd();

    glBegin(GL_LINE_LOOP);
         glVertex3f(_bbox[1], _bbox[3], _bbox[5] );
         glVertex3f(_bbox[0], _bbox[3], _bbox[5] );
         glVertex3f(_bbox[0], _bbox[2], _bbox[5] );
         glVertex3f(_bbox[1], _bbox[2], _bbox[5] );
    glEnd();

    glBegin(GL_LINES);
        glVertex3f(_bbox[1], _bbox[3], _bbox[4] );
        glVertex3f(_bbox[1], _bbox[3], _bbox[5] );
    
        glVertex3f(_bbox[0], _bbox[3], _bbox[4] );
        glVertex3f(_bbox[0], _bbox[3], _bbox[5] );
    
        glVertex3f(_bbox[0], _bbox[2], _bbox[4] );
        glVertex3f(_bbox[0], _bbox[2], _bbox[5] );
    
        glVertex3f(_bbox[1], _bbox[2], _bbox[4] );
        glVertex3f(_bbox[1], _bbox[2], _bbox[5] );
    glEnd();

    if( smooth_lines() ) {
        glDisable (GL_LINE_SMOOTH);
        glDisable (GL_BLEND);
    }


    if( slights.some_active() ) {
        glEnableClientState(GL_NORMAL_ARRAY);
        slights.gl_enable_all();
    }
}




