/***************************************************************************
                          base_object.h  -  description
                             -------------------
    begin                : Fri Dec 10 1999
    copyright            : (C) 1999 by Juan Pablo Romero Mndez
    email                : juan_pablo_22@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _BASE_OBJECT_H
#define _BASE_OBJECT_H

#include "sup_global.h"
#include "util.h"
#include "svector.h"
#include "sarray.h"
#include "light_material.h"
#include "lights.h"

#include <glib.h>
#include <parser.h>
#include <stdio.h>
#include <string>
#include <GL/gl.h>
#include <deque>



typedef deque<GNode*> GnodeList;

typedef GnodeList::iterator                Iter;
typedef GnodeList::reverse_iterator       rIter;
typedef deque<Transformations>::iterator trIter;


// This class maintains the geometry and certain state variables
// of an actual object.

class BaseObject
{
    friend class Scene;
    friend class Multiple;

protected:

    SMaterial           _material;
    string                  _name;
    GLubyte*   _transparent_array;

    VIRTUAL_ATTRIBUTE(smooth,          gboolean);     
    VIRTUAL_ATTRIBUTE(idle,            gboolean);     
    VIRTUAL_ATTRIBUTE(motion,          gboolean);     
    VIRTUAL_ATTRIBUTE(mono,            gboolean);     
    VIRTUAL_ATTRIBUTE(normals,         gboolean);     
    VIRTUAL_ATTRIBUTE(selected,        gboolean);     
    VIRTUAL_ATTRIBUTE(show_bbox,       gboolean);     
    VIRTUAL_ATTRIBUTE(show_mesh,       gboolean);     
    VIRTUAL_ATTRIBUTE(mesh_smooth,     gboolean);     
    VIRTUAL_ATTRIBUTE(smooth_lines,    gboolean);     
    VIRTUAL_ATTRIBUTE(transparent,     gboolean);     
    VIRTUAL_ATTRIBUTE(invisible  ,     gboolean);
    VIRTUAL_ATTRIBUTE(smooth_edges,    gboolean);     
    VIRTUAL_ATTRIBUTE(frame_line_width,  gfloat);
    VIRTUAL_ATTRIBUTE(lines_width,       gfloat);     
    VIRTUAL_ATTRIBUTE(points_width,      gfloat);     
    VIRTUAL_ATTRIBUTE(polygon_mode, PolygonMode);

    ATTRIBUTE(last_motion,              gdouble);
    ATTRIBUTE(previous_last_motion,     gdouble);
    ATTRIBUTE(gnode,                     GNode*);
    ATTRIBUTE(type,                  ObjectType);
    ATTRIBUTE(gl_load_name,                gint);

    SVECTOR_ATTRIBUTE(mono_color,   4);
    SVECTOR_ATTRIBUTE(frame_color,  4);
    SVECTOR_ATTRIBUTE(normal_color, 4);
    SVECTOR_ATTRIBUTE(mesh_color,   4);

    SVECTOR_ATTRIBUTE(bbox,       6);

public:

    BaseObject();
    virtual ~BaseObject              () {}

    virtual void   draw            (void)             {}
    virtual void   draw_bbox       (void);
    virtual gfloat farest_vertex   (gfloat* center)   {}
    virtual void   calc_center     ()                 {}
    virtual void   update_bbox       ()                 {}

    virtual void parse_xml_node(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) {}
    virtual void initialize();

    virtual void offset  (gfloat * off)  {}
    virtual void lastquat(gfloat * quat) {}
    virtual void curquat (gfloat * quat) {}
    virtual void center  (gfloat * c)    {}
    virtual SVector<3> &  center  () {}

    virtual void delete_empty_transformations() {}

    void name                    (gchar* str) { _name = string(str ? str : ""); }
    void transparent_array       (gint n)     { _transparent_array = patron[n]; }
    const gchar *name            ()           { return _name.c_str();           }
    string       name_string     ()           { return _name; }

    guint depth(void) { return g_node_depth(_gnode); }
    BaseObject * parent() { return BASE_OBJECT(_gnode->parent); }

};


#endif

