/***************************************************************************
 gtk-sup.h  -  description
 -------------------
 begin                : Fri Dec 10 1999
 copyright            : (C) 1999 by Juan Pablo Romero Mndez
 email                : juan_pablo_22@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __GTK_SUP_H
#define __GTK_SUP_H


#include "superficie.h"

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <gtkgl/gtkglarea.h>
#include <gtkgl/gdkgl.h>
#include <gdk_imlib.h>

class GtkScene;

extern gint _gtk_private_redraw(GtkScene* scene);

/* The definition of  '_gtk_private_redraw' must be:
 *
 *  gint _gtk_private_redraw(GtkScene* scene) {
 *      scene->redraw();
 *  }
 *
 * It *has* to be somewhere in the aplication.
 */

class GtkScene : public Scene {
    GtkWidget* glarea;
    gint idle_tag;

    gdouble _last_event_x, _last_event_y;
    gboolean _b1Q, _b2Q, _b3Q, _ctrlQ;


public:

    gint _event_button;

    GtkScene() : glarea(NULL) {
        //Just to generate a valid idle_tag, so glib don't complain the first time.
        add_idle(); remove_idle();
    }

    void add_glarea(GtkWidget* g) { glarea = g; }

    gboolean make_current_gl() { return glarea ? gtk_gl_area_make_current(GTK_GL_AREA(glarea)) : FALSE; }
    void begin_gl() { gtk_gl_area_begingl(GTK_GL_AREA(glarea)); }
    void end_gl  () { gtk_gl_area_endgl(GTK_GL_AREA(glarea));   }

    virtual gint event_x() { return (gint) _last_event_x; }
    virtual gint event_y() { return (gint) _last_event_y; }
    virtual gint event_button() { return _event_button; }

    virtual gboolean button1() { return _b1Q; }
    virtual gboolean button2() { return _b2Q; }
    virtual gboolean button3() { return _b3Q; }
    virtual gboolean control() { return _ctrlQ; }

    virtual gint window_width() { return glarea ? glarea->allocation.width : 0; }
    virtual gint window_height() { return glarea ? glarea->allocation.height : 0; }

    virtual void ui_notify_gnode_selected(GNode* gnode) {}

    void remove_idle() {
        gtk_idle_remove( idle_tag );
    }

    void add_idle() {
        idle_tag = gtk_idle_add((GtkFunction) _gtk_private_redraw, this);
    }

    void redraw() {
        if (gtk_gl_area_make_current(GTK_GL_AREA(glarea))) {
            draw();
            gtk_gl_area_swapbuffers(GTK_GL_AREA(glarea));
        }

    }

    void update_state(void* _event) {

        GdkModifierType state;
        GdkEventButton* event = (GdkEventButton*) _event;

        _last_event_x = event->x;
        _last_event_y = event->y;

        if( ((GdkEventMotion*)_event)->is_hint )
            gdk_window_get_pointer(event->window,
                                   (gint*) &_last_event_x,
                                   (gint*) &_last_event_y,
                                   &state);

        _b1Q = state & GDK_BUTTON1_MASK;
        _b2Q = state & GDK_BUTTON2_MASK;
        _b3Q = state & GDK_BUTTON3_MASK;
        _ctrlQ = state & GDK_CONTROL_MASK;
    }

    void save_as(gchar* file) { }

};

#endif