/***************************************************************************
                          lights.cc  -  description
                             -------------------
    begin                : Fri Dec 10 1999
    copyright            : (C) 1999 by Juan Pablo Romero Mndez
    email                : juan_pablo_22@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <lights.h>

static typeof(GL_LIGHT0) _tmp_gl_lights[8] = {
    GL_LIGHT0, GL_LIGHT1, GL_LIGHT2, GL_LIGHT3, GL_LIGHT4,
    GL_LIGHT5, GL_LIGHT6, GL_LIGHT7
};


SceneLights::SceneLights()
{
    make_defaults();
    S_GL_LIGHTS = _tmp_gl_lights;
}

SceneLights::~SceneLights() { }

void
SceneLights::make_defaults()
{
    _slights.push_back( SLight() );
    _slights.push_back( SLight() );
    _slights.push_back( SLight() );

    _slights[0].position( SVector<4>(1, 0, 1, 0) );
    _slights[1].position( SVector<4>(1, 1, 1, 0) );
    _slights[2].position( SVector<4>(0, 1, 1, 0) );

    _slights[0].diffuse( SVector<4>(1, 0, 0, 1) );
    _slights[1].diffuse( SVector<4>(0, 1, 0, 1) );
    _slights[2].diffuse( SVector<4>(0, 0, 1, 1) );

    _slights[0].active(TRUE);
    _slights[1].active(TRUE);
    _slights[2].active(TRUE);
}

void
SceneLights::enable_all()
{
    glEnable(GL_LIGHTING);
    for(gint i = 0; i < _slights.size(); i++ ) {
        _slights[i].active(TRUE);
        glEnable(S_GL_LIGHTS[i]);
    }
}

// only opengl stuff

void
SceneLights::gl_enable_all()
{
    glEnable(GL_LIGHTING);
    for(gint i = 0; i < _slights.size(); i++ ) {
        if( _slights[i].active() )
            glEnable(S_GL_LIGHTS[i]);
    }
}


void
SceneLights::disable_all()
{
    glDisable(GL_LIGHTING);
    for(gint i = 0; i < _slights.size(); i++ ) {
        _slights[i].active(FALSE);
        glDisable(S_GL_LIGHTS[i]);
    }
}

// only opengl stuff
void
SceneLights::gl_disable_all()
{
    glDisable(GL_LIGHTING);
    for(gint i = 0; i < _slights.size(); i++ ) {
        glDisable(S_GL_LIGHTS[i]);
    }
}

void
SceneLights::erase_all()
{
    gl_disable_all();
    deque<SLight>().swap( _slights );
}

void
SceneLights::erase(gint light)
{
    g_assert( light < _slights.size() );

    // disable the last light
    glDisable(S_GL_LIGHTS[_slights.size() - 1]);
    _slights.erase( _slights.begin() + light );

    update_all();
}

void
SceneLights::push_back()
{
    g_assert( _slights.size() < MAX_LIGHTS );
    glEnable(S_GL_LIGHTS[_slights.size()]);
    _slights.push_back( SLight() );

    update_all();
}

gboolean
SceneLights::some_active()
{
    for(gint i=0; i < _slights.size(); i++ ) {
        if( _slights[i].active() )
            return TRUE;
    }

    return FALSE;
}

void
SceneLights::update_all()
{
    update_ambient();
    update_diffuse();
    update_specular();
}

void
SceneLights::update_position()
{
    for(gint i = 0; i < _slights.size(); i++ ) {
        glLightfv(S_GL_LIGHTS[i], GL_POSITION, & _slights[i].position());
    }
}

void
SceneLights::update_ambient()
{
    for(gint i = 0; i < _slights.size(); i++ ) {
        glLightfv(S_GL_LIGHTS[i], GL_AMBIENT, & _slights[i].ambient());
    }
}


void
SceneLights::update_diffuse()
{
    for(gint i = 0; i < _slights.size(); i++ ) {
        glLightfv(S_GL_LIGHTS[i], GL_DIFFUSE, & _slights[i].diffuse());
    }
}

void
SceneLights::update_specular()
{
    for(gint i = 0; i < _slights.size(); i++ ) {
        glLightfv(S_GL_LIGHTS[i], GL_SPECULAR, & _slights[i].specular());
    }
}


void
SceneLights::update_ambient(gint light, gfloat* val)
{
    g_assert( light < _slights.size() );

    if( val != NULL )
        _slights[light].ambient(val);

    glLightfv(S_GL_LIGHTS[light], GL_AMBIENT, & _slights[light].ambient());
}

void
SceneLights::update_position(gint light, gfloat* val)
{
    g_assert( light < _slights.size() );
    _slights[light].position(val);
}

void
SceneLights::update_diffuse(gint light, gfloat* val)
{
    g_assert( light < _slights.size() );

    if( val != NULL )
        _slights[light].diffuse(val);

    glLightfv(S_GL_LIGHTS[light], GL_DIFFUSE, & _slights[light].diffuse());
}

void
SceneLights::update_specular(gint light, gfloat* val)
{
    g_assert( light < _slights.size() );

    if( val != NULL )
        _slights[light].specular(val);

    glLightfv(S_GL_LIGHTS[light], GL_SPECULAR, & _slights[light].specular());
}

void
SceneLights::update_active(gint light, gboolean val)
{
    g_assert( light < _slights.size() );
    _slights[light].active(val);

    val ? glEnable(S_GL_LIGHTS[light]) : glDisable(S_GL_LIGHTS[light]);
}


