/***************************************************************************
                          multiple.h  -  description
                             -------------------
    begin                : Fri Dec 10 1999
    copyright            : (C) 1999 by Juan Pablo Romero Mndez
    email                : juan_pablo_22@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef _MULTIPLE_H
#define _MULTIPLE_H


#include "base_object.h"
#include "object.h"
#include "sup_global.h"
#include "transformations.h"

#include <deque>

// Esta clase lo nico que hace es controlar ciertas
// variables que afectan a todos los nodos hijos;
// as como proveer ciertas funciones que permitan
// manejar este nodo como si fuera un objeto real.
// Envuelve ciertas funciones de GNode.


class Multiple : public BaseObject
{

public:

    static gint      _selection_name_stack;
    static gboolean           _select_mode;
    static GnodeList _selection_gnode_list;


    Multiple();

    virtual     ~Multiple       ();
    virtual void draw           (void);
    virtual void update_bbox      ();

    virtual void offset  (gfloat * off);
    virtual void lastquat(gfloat * quat);
    virtual void curquat(gfloat * quat);
    virtual void center(gfloat * c);

    virtual SVector<3> & center  ();

    gboolean    some_selected   ();
    gboolean    some_motion     ();
    gboolean    some_idle       ();
    Object*     some_child      ();

    void        translate_to            (gfloat * point );
    void        apply_transformations   (Object * ob    );
    gfloat      farest_vertex           (gfloat * center);
    BaseObject* part                    (gint     n     );

    void parse_xml_node(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur);
    void initialize();


    gint        length                  ()       { return g_node_n_children(_gnode); }
    gboolean    is_empty                ()       { return length()==0; }
    BaseObject* operator[]              (gint n) { return part(n); }

    GNode*       append        (BaseObject* ob);
    GNode*         move          (GNode* node);
    virtual void remove          (GNode* node);
    virtual void unlink          (GNode* node);


    virtual void calc_center    ();
    virtual void smooth         (gboolean s);
    virtual void mono           (gboolean s);
    virtual void normals        (gboolean s);
    virtual void transparent    (gboolean s);
    virtual void mono_color     (gdouble* v);
    virtual void show_mesh      (gboolean s);
    virtual void mesh_smooth   (gboolean s);
    virtual void smooth_lines   (gboolean s);
    virtual void show_bbox_all  (gboolean s);
    virtual void polygon_mode   (PolygonMode s);

    virtual void lines_width    (gfloat   v);
    virtual void points_width   (gfloat   v);
    virtual void smooth_edges   (gboolean s);
    virtual void frame_line_width  (gfloat v);

    // set idle on all children
    void         idle_all        (gboolean s);

} ;


#endif






