/***************************************************************************
                          object.h  -  description
                             -------------------
    begin                : Fri Dec 10 1999
    copyright            : (C) 1999 by Juan Pablo Romero Mndez
    email                : juan_pablo_22@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _OBJECT_H
#define _OBJECT_H

#include "sup_global.h"
#include "util.h"
#include "base_object.h"
#include "sarray.h"
#include "transformations.h"

#include <glib.h>
#include <stdio.h>
#include <string>
#include <GL/gl.h>

class Object : public BaseObject
{
    friend class Scene;
    friend class Multiple;
    friend gboolean _lastquat(GNode * gnode, pair<GNode*, gfloat*> * data);


protected:
    
    VectorArray<gfloat> _vertex;
    VectorArray<gfloat> _color;

    deque<Transformations> _trans;
    // the bboxes in the coordinates of each parent
    deque< SVector<6> >   _bboxes;

    static PolygonMode _polygon_mode_up;
    static PolygonMode _polygon_mode_down;

public:

    Object();

    static void      polygon_mode_down(PolygonMode pm) { _polygon_mode_down = pm; }

    virtual         ~Object          ();
    virtual void     draw            (void          );
    virtual gfloat   farest_vertex   (gfloat* center);
    virtual void     calc_center     ();
    virtual void     update_bbox       ();

    virtual void offset  (gfloat * off);
    virtual void lastquat(gfloat * quat);
    virtual void curquat(gfloat * quat);
    virtual void center  (gfloat * c);

    virtual void delete_empty_transformations();

    virtual gfloat*  offset  () { return & _trans.back()._offset; }
    virtual gfloat*  lastquat() { return & _trans.back()._lastquat; }
    virtual gfloat*  curquat () { return & _trans.back()._curquat; }
    virtual SVector<3> & center  () { return  _trans.back()._center; }

    Transformations* get_transf_by_node(GNode* n);
    GNode* moving_some_parent();
    gboolean some_parent_idle(GnodeList & idle_parents);

    deque<Transformations> & trans () { return _trans;  }
    deque< SVector<6> >    & bboxes() { return _bboxes; }

    void apply_transformations   ();
    void local_to_scene(GNode* parent, gfloat* p, gfloat* pp);
    void update_bboxes();

    gfloat *         vertex          () { return _vertex.begin(); }
    gint             vertex_size     () { return _vertex.size();  }
    
    void             set_color_and_material_state();
    void parse_xml_node(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur);

};


#endif
