/***************************************************************************
                          polygon.cc  -  description
                             -------------------
    begin                : Fri Dec 10 1999
    copyright            : (C) 1999 by Juan Pablo Romero Mndez
    email                : juan_pablo_22@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "polygon.h"
#include "read_data.h"
#include "util.h"

#include <GL/gl.h>
#include <strstream>

using superficie::minus;
using superficie::plus;

Polygon::Polygon()
{
    _type  = POLYGON;
}

Polygon::~Polygon() {}

#if 0

void
Polygon::draw(void)
{
    Object::draw();
    glNormalPointer(GL_FLOAT, 0, _normal.ptr() );

    glDrawElements(GL_POLYGON, _index.size(), GL_UNSIGNED_INT, _index.begin());

    /*
     glBegin(GL_POLYGON);

     for(gint i=0; i<_vertex.size(); i++)
     glVertex3fv(_vertex[i]);

     glEnd();
     */
}



#else

void
Polygon::draw(void)
{

    Object::draw();

    glNormalPointer(GL_FLOAT, 0, _normal.ptr() );

    //Enable line smooth. Do it here to avoid repetition below.
    if( _polygon_mode != POLY_POINT ) {
        glLineWidth (lines_width ());
        if( smooth_lines() ) {
            glEnable (GL_LINE_SMOOTH);
            glEnable (GL_BLEND);
            glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        }
    }

    if( show_mesh() ) {
        glEnable(GL_POLYGON_OFFSET_FILL);
        glPolygonOffset(1.0, 1.0);
    }

#if 1
    glBegin(GL_POLYGON);

    for(gint i=0; i<_vertex.size(); i++)
        glVertex3fv(_vertex[i]);

    glEnd();

#else

    glDrawElements(GL_POLYGON, _index.size(), GL_UNSIGNED_INT, _index.begin());

#endif

    /* This will obscure the frame */
    if( smooth_edges() ) {
        glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
        glDrawElements(GL_POLYGON, _index.size(), GL_UNSIGNED_INT, _index.begin());
    }


    if( show_mesh() ) {
        glDisable(GL_POLYGON_OFFSET_FILL);

        /* Si deshabilito la iluminacin aqu, para restaurarla
         * al estado original necesitara conocer el estado de
         * TheScene, o bien leer el estado de OpenGL; pero creo
         * que esto es bastante costoso. As que por lo pronto,
         * dibujamos la malla con iluminacin.
         */

        // Creo que esto no es muy rpido.
        //glPushAttrib(GL_LINE_BIT|GL_COLOR_BUFFER_BIT);

        if( mesh_smooth() ) {
            glEnable (GL_LINE_SMOOTH);
            glEnable (GL_BLEND);
            glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
            //glBlendFunc (GL_SRC_ALPHA, GL_ONE);
        }
        else {
            glDisable (GL_LINE_SMOOTH);
            glDisable (GL_BLEND);
        }

        glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );

        { // disable lighting
            glDisableClientState(GL_COLOR_ARRAY);
            glDisable(GL_COLOR_MATERIAL);
            slights.gl_disable_all();
            glDisableClientState(GL_NORMAL_ARRAY);
            glColor3fv( & _mesh_color );
        }

        glLineWidth (frame_line_width());
        glDrawElements(GL_POLYGON, _index.size(), GL_UNSIGNED_INT, _index.begin());

        //glPopAttrib();

        if( mesh_smooth() && !smooth_lines() ) {
            glDisable (GL_LINE_SMOOTH);
            glDisable (GL_BLEND);

        }
        else if( !mesh_smooth() && smooth_lines() ) {
            glEnable (GL_LINE_SMOOTH);
            glEnable (GL_BLEND);
            glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
            //glBlendFunc (GL_SRC_ALPHA, GL_ONE);
        }

        if( slights.some_active() ) {
            glEnableClientState(GL_NORMAL_ARRAY);
            slights.gl_enable_all();
        }
    }

    if( _normals ) {
        { // disable lighting
            glDisableClientState(GL_COLOR_ARRAY);
            glDisable(GL_COLOR_MATERIAL);
            slights.gl_disable_all();
            glDisableClientState(GL_NORMAL_ARRAY);
            glColor3fv( & _normal_color );
        }

        glLineWidth (lines_width ());
        draw_normals();

        if( slights.some_active() ) {
            glEnableClientState(GL_NORMAL_ARRAY);
            slights.gl_enable_all();
        }
    }


    if( _polygon_mode != POLY_POINT ) {
        if( smooth_lines() ) {
            glDisable (GL_LINE_SMOOTH);
            glDisable (GL_BLEND);
        }
    }
}

#endif


void
Polygon::parse_xml_node(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) {}

void
Polygon::initialize()
{
    //TODO: fix this!!!
    static guint _loaded = 0;
    ostrstream n;
    n << "polygon" << _loaded++ << ends;
    _name = n.str();
    n.freeze(0);
    //

    _index = vector<guint>(_vertex.size());
    _for_each(i, _index.size(), _index[i] = i;);
    _tangente = _normal = VectorArray<gfloat>(_vertex.size());
    generate_normal_vectors();
    update_normal_tangent_vectors();
}




void
Polygon::update_normal_tangent_vectors(void)
{
    for(gint i = 0; i < _tangente.size(); i++)
        plus( _tangente[i] , _vertex[i], _normal[i] );
}


void
Polygon::generate_normal_vectors(void)
{
    SVector<3> v1, v2, v3;
    gint i;
    /* &v is just v.ptr() */

    //_normal[i] = (_vertex[i-1] - _vertex[i]) * (_vertex[i] - _vertex[i+1]);

    { // first vertex
        i = 0;
        minus( &v1, _vertex[ _vertex.size() - 1], _vertex[i] );
        minus( &v2, _vertex[i],  _vertex[i+1] );
        cross( _normal[i], &v1, &v2 );
        normalize( _normal[i] );
    }


    for(i=1; i<_normal.size() - 1; i++) {
        minus( &v1, _vertex[i-1], _vertex[i  ] );
        minus( &v2, _vertex[i  ],  _vertex[i+1] );
        cross( _normal[i], &v1, &v2 );
        normalize( _normal[i] );
    }


    { // last vertex
        i = _vertex.size() - 1;
        minus( &v1, _vertex[i-1], _vertex[i] );
        minus( &v2, _vertex[i],  _vertex[0] );
        cross( _normal[i], &v1, &v2 );
        normalize( _normal[i] );
    }
}

void
Polygon::draw_normals()
{
    for(gint i = 0; i < _vertex.size(); i ++) {
        glBegin(GL_LINES);
        glVertex3fv(_vertex[i]);
        glVertex3fv(_tangente[i]);
        glEnd();
    }
}
