/***************************************************************************
                          sarray.h  -  description
                             -------------------
    begin                : Fri Dec 10 1999
    copyright            : (C) 1999 by Juan Pablo Romero Mndez
    email                : juan_pablo_22@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef _SARRAY_H
#define _SARRAY_H


#include "util.h"
#include "svector.h"

template <typename T>
class VectorArray
{
    T * _array;

    // _length is the number of "vectors":
    // _length = _array_length / 3

    gint _length;
    gint _array_length;

public:

    VectorArray() : _length(0) {
        _array_length = 0;
        _array = NULL;
    }


    VectorArray(gint l) : _length(l) {
        _array_length = l*3;
        _array = new T[_array_length];
        _for_each( i, _array_length, _array[i] = 0; );
    }

    VectorArray(const VectorArray<T> & a) {
        g_assert(_length == a._length);
        _for_each(i, _array_length, _array[i] = a._array[i]; );
    }

    template<typename TP>
    VectorArray(const TP * a_array, const gint a_length) {
        _array_length = a_length;
        _length = a_length / 3;
        _array = new T[_array_length];
        _for_each(i, _array_length, _array[i] = (T) a_array[i];);
    }

    VectorArray<T> & operator = (const VectorArray<T> & a) {
        _length = a._length;
        _array_length = a._array_length;
        _array = new T[_array_length];
        _for_each(i, _array_length, _array[i] = a._array[i];);
        return *this;
    }

    ~VectorArray() { delete _array; }

    gint size() { return _length; }

    T * operator[] (gint n) {
            g_assert(n < _length);
        return &_array[n * 3];
    }

    T * ptr() { return _array; }
    T * begin() { return _array; }
    T * operator & () { return _array; }

    T max_x() { return max_coord(0); }
    T max_y() { return max_coord(1); }
    T max_z() { return max_coord(2); }

    T min_x() { return min_coord(0); }
    T min_y() { return min_coord(1); }
    T min_z() { return min_coord(2); }

    T max_coord(gint offset);
    T min_coord(gint offset);
};


template <typename T>
T VectorArray<T>::max_coord(gint offset)
{
    T max = _array[offset];

    for (gint i = offset; i < _array_length; i+=3)
        if (_array[i] > max) max = _array[i];

    return max;
}

template <typename T>
T VectorArray<T>::min_coord(gint offset)
{
    T min = _array[offset];

    for (gint i = offset; i < _array_length; i+=3)
        if (_array[i] < min) min = _array[i];

    return min;
}

#endif