/***************************************************************************
                          scene.h  -  description
                             -------------------
    begin                : Fri Dec 10 1999
    copyright            : (C) 1999 by Juan Pablo Romero Mndez
    email                : juan_pablo_22@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef _SCENE_H
#define _SCENE_H

#include "multiple.h"
#include "svector.h"
#include "light_material.h"

#include <glib.h>
#include <parser.h>

class Scene : public Multiple
{

protected:

    ATTRIBUTE(motion,                  gint);
    ATTRIBUTE(axis,                    gint);
    ATTRIBUTE(razon,                 gfloat);
    ATTRIBUTE(fovy,                  gfloat);
    ATTRIBUTE(beginx,                gfloat);
    ATTRIBUTE(beginy,                gfloat);
    ATTRIBUTE(timer,                GTimer*);
    ATTRIBUTE(near,                  gfloat);
    ATTRIBUTE(far,                   gfloat);
    ATTRIBUTE(normalize,               gint);
    ATTRIBUTE(two_side_lighting,       gint);
    ATTRIBUTE(viewpoint_inc,         gfloat);
    ATTRIBUTE(animation,           gboolean);
    ATTRIBUTE(button_down,         gboolean);
    ATTRIBUTE(no_light_when_moving,gboolean);
    ATTRIBUTE(feedBufferSize,        gulong);

    // TODO: use svector instead of array

    ARRAY_ATTRIBUTE(viewpoint,      3);
    SVECTOR_ATTRIBUTE(bg_color,     4);

private:

    gint   _update_object_vars;
    GNode *          _selected_gnode;

    gint         _normals_type;
    gfloat              _scale;
    gfloat              _angle;
    gfloat              _x_pos;
    gfloat              _y_pos;
    gfloat              _z_pos;


public:

    Scene();
    ~Scene();

    void draw(void);

    virtual gint        event_x () {}
    virtual gint        event_y () {}
    virtual void        add_idle() {}
    virtual void        redraw  () {}
    virtual gboolean    button1 () {}
    virtual gboolean    button2 () {}
    virtual gboolean    button3 () {}
    virtual gboolean    control () {}
    virtual gint        event_button () {}
    virtual void        remove_idle  () {}
    virtual gint        window_width () {}
    virtual gint        window_height() {}
    virtual void        ui_notify_gnode_selected(GNode* gnode) {}
    virtual void        save_as         (gchar* file) {}

    virtual void        remove          (GNode* node);
    
    void button_press();
    void button_release();
    void motion_notify();
    void window_reshape();


    BaseObject * selected_object (void) { return BASE_OBJECT(_selected_gnode); }
    void        selected_node   (GNode* node);
    GNode *     selected_node   () { return _selected_gnode; }

    BaseObject* operator*       () { return selected_object(); }

    void        update_viewpoint(void);
    void        update_transformations(void);
    void        realize         (void);
    gint        read            (gchar* file, gchar* buffer, gint size);
    void        select          (void);
    void        print           (gchar* file);

    void        set_from_1st_object(void);


    xmlDocPtr parse_xml_file(gchar* file);
    void parse_xml_doc(xmlDocPtr doc);
    BaseObject * parse_xml_object(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur);
    void parse_xml_object_group(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur, Multiple* root);

    gint read_and_parse_traditional(gchar* file, FILE* fp);

};


#endif
