/***************************************************************************
                          surface.cc  -  description
                             -------------------
    begin                : Fri Dec 10 1999
    copyright            : (C) 1999 by Juan Pablo Romero Mndez
    email                : juan_pablo_22@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "surface.h"
#include "read_data.h"
#include <strstream>


Surface::Surface() {
    _type = QUADS;
}

void
Surface::parse_xml_node(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
{
    cur = cur->childs;

    while (cur != NULL) {
        if( EQSTR(cur->name, "Index") && (cur->ns == ns)) {
            parse_array(_quad_index,
                        (gchar*) xmlNodeListGetString(doc, cur->childs, 1),
                        (gchar*) xmlGetProp(cur, "Length"));
        }
        cur = cur->next;
    }
}


void
Surface::initialize()
{
    //TODO: fix this!!!
    static guint _loaded = 0;
    ostrstream n;
    n << "quads" << _loaded++ << ends;
    _name = n.str();
    n.freeze(0);
    //

    _index = vector<guint>(_quad_index.size() * 1.5);
    _hline_index = vector<guint>(_quad_index.size());
    split_quads();
    _triangle_number = _index.size() / 3;
    _triangle_normal = VectorArray<gfloat>( _triangle_number );

    _tangente = _normal = VectorArray<gfloat>(_vertex.size());
    t_message.generating_normals = TRUE;
    normal_per_triangle();
    normal_per_vertex(90);
    update_normal_tan_vecs();
}

void
Surface::draw_faces(void)
{
    glDrawElements(GL_QUADS, _quad_index.size(),
                   GL_UNSIGNED_INT,_quad_index.begin());
}


void
Surface::draw_edges(void)
{
    glDrawElements(GL_QUADS, _quad_index.size(),
                   GL_UNSIGNED_INT,_quad_index.begin());
}

void
Surface::draw_mesh(void)
{
    glDrawElements(GL_QUADS, _quad_index.size(),
                   GL_UNSIGNED_INT,_quad_index.begin());
}

void
Surface::draw_lines(void)
{
    glDrawElements(GL_QUADS, _quad_index.size(),
                   GL_UNSIGNED_INT,_quad_index.begin());
}

void
Surface::draw_vlines(void)
{
    glDrawElements(GL_LINES, _quad_index.size(),
                   GL_UNSIGNED_INT,_quad_index.begin());
}

void
Surface::draw_hlines(void)
{
    glDrawElements(GL_LINES, _quad_index.size(),
                   GL_UNSIGNED_INT,_hline_index.begin());
}

/*
 * given an array of quads indexes, generates the
 * indexes for triangles.
 */

void
Surface::split_quads()
{
    gint i;
    
    /*
     *      split each quad in 2 triangles:

     *                     [#]  0  1  2    3  4  5
                                |  |  |    |  |  |
     *      {0, 1, 2, 3} ---> {{0, 1, 3}, {1, 2, 3}}
     
            3----<---2       3----<---2
            |        |       |       |
            v        ^   =>  v   .    ^
            |        |       |       |
            0---->---1       0---->---1
     * 
     *      the position in corresponding [vertex] arrays is:
     * 
     *      quad_index               triangle_index
     *      ----------               --------------
     *      0-3                      0-5
     *      4-7                      6-11
     *      8 ...                    8+4...
     *      ...                      ...
     *      4n ...                   4n + 2n = 6n
     */

    for(i = 0; i < _quad_index.size() / 4; i++) {
        _index[6*i + 0] = _quad_index[4*i + 0];
        _index[6*i + 1] = _quad_index[4*i + 1];
        _index[6*i + 2] = _quad_index[4*i + 3];
                                               
        _index[6*i + 3] = _quad_index[4*i + 1];
        _index[6*i + 4] = _quad_index[4*i + 2];
        _index[6*i + 5] = _quad_index[4*i + 3];


        _hline_index[4*i + 0] = _quad_index[4*i + 1];
        _hline_index[4*i + 1] = _quad_index[4*i + 2];
        _hline_index[4*i + 2] = _quad_index[4*i + 3];
        _hline_index[4*i + 3] = _quad_index[4*i + 0];
    }
}

