/***************************************************************************
 svector.h  -  description
 -------------------
 begin                : Fri Dec 10 1999
 copyright            : (C) 1999 by Juan Pablo Romero Mndez
 email                : juan_pablo_22@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _SVECTOR_H
#define _SVECTOR_H

#include "util.h"


template <gint SIZE, typename T = float>
    class SVector
{
    T _vector[SIZE];

public:
    SVector()  {
        _for_each( i, SIZE, _vector[i] = 0; );
    }

    template<typename TP>
        SVector( TP * v )  {
            _for_each( i, SIZE, _vector[i] = (T) v[i]; );
        }

    SVector(const SVector<SIZE, T> & v) {
        _for_each(i, SIZE, _vector[i] = v._vector[i]; );
    }

    SVector<SIZE, T> & operator = (const SVector<SIZE, T> & v) {
        _for_each(i, SIZE, _vector[i] = v._vector[i];);
        return *this;
    }

    gboolean operator == (const SVector<SIZE, T> & v) {
        for(gint i=0; i < SIZE; i++) {
            if( _vector[i] != v._vector[i] )
                return FALSE;
        }
        return TRUE;
    }

    SVector( T v0, T v1, T v2, T v3) {
        g_assert( SIZE >= 4 );
        _vector[0] = v0;
        _vector[1] = v1;
        _vector[2] = v2;
        _vector[3] = v3;
    }

    SVector( T v0, T v1, T v2) {
        g_assert( SIZE >= 3 );
        _vector[0] = v0;
        _vector[1] = v1;
        _vector[2] = v2;
    }


    template<typename TP>
        SVector<SIZE, T> & operator = (TP * v) {
            _for_each(i, SIZE, _vector[i] = (T) v[i];);
            return *this;
        }

    T * ptr   ()  { return _vector; }
    T * operator & () { return _vector; }

    gint length()  { return SIZE; }
    gint size()  { return SIZE; }

    T & part  (gint i) { return _vector[i]; }
    T & operator[] (gint i) { return _vector[i]; }

    SVector<SIZE, T> & operator += (const SVector<SIZE, T> & v) {
        _for_each(i, SIZE, _vector[i] += v._vector[i];);
        return *this;
    }

    void print(ostream & os) {
        _for_each(i, SIZE, os << _vector[i] << " "; );
        os << endl;
    }
};



#endif