/***************************************************************************
                          triangles.cc  -  description
                             -------------------
    begin                : Fri Dec 10 1999
    copyright            : (C) 1999 by Juan Pablo Romero Mndez
    email                : juan_pablo_22@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "triangles.h"
#include "util.h"
#include "svector.h"
#include "trackball.h"
#include "read_data.h"

#include <GL/gl.h>
#include <strstream>

using superficie::minus;
using superficie::plus;


Triangles::Triangles()
{
    _type = TRIANGLES;
}

Triangles::~Triangles()
{
    
}

/* Fills 'lista' with the indices of the triangles the given vertex ('i') is in.
 * Returns the number of such triangles
 */

gint
Triangles::find_triangles_contaning(guint i, vector<guint> & lista)
{
    gint n=0, k=0;

    while( n < _index.size() ) {
        if(_index[n] == i) {
            lista[k++] = triangle_number(n);
            //Jump to the next multiple of 3 (begining of the next triangle)
            n = (n/3 + 1) * 3;
        }
        else
            n++;
    }

    return k;
}


void
Triangles::normal_per_vertex(gfloat angle)
{
    //angle will be used later

    // see above.
    // the max size lista can grow ever is _vertex.size(),
    // ie, the vertex is in all the triangles.

    // TODO: optimize this function.
    gint t_found;
    vector<guint> lista(_vertex.size());;
    t_message.triangle_total = _vertex.size();

    for(gint i=0; i < _vertex.size(); i++) {

        { // allow aborting
            if( t_message.abort_loading )
                longjmp(main_loop, -1);
            t_message.triangle_procesed = i;
        }

        // find what triangles the i-th vertex is in.
        t_found = find_triangles_contaning(i, lista);

        // sum the triangle normals
        // The value of the normal of a give vertex is just the sum of
        // the normals of the triangles that contain this vertex
        // TODO: find a better algorithm
        // We should zero _normal[i] before begin to sum

        for(gint j=0; j<t_found; j++)
            sum(_normal[i], _triangle_normal[lista[j]]);

        normalize( _normal[i] );
    }
}



void
Triangles::normal_per_triangle(void)
{
    SVector<3> v1, v2;
    gint k, i;

    

    //             t1
    //            / |
    //           /  |
    //          /   ^
    //         /    |
    //        /     |
    //      t0 ->-- t1
    //      
    //      c = (t0 - t1) x (t1 - t2)
    //      n = c / |c|
    //

    /* Calculate one normal per triangle */
    for( k = i = 0; i < _triangle_normal.size(); i++, k = 3 * i) {

        // v1 = (t0 - t1)
        minus( &v1, _vertex[_index[k  ]], _vertex[_index[k+1]] );
        // v2 = (t1 - t2)
        minus( &v2, _vertex[_index[k+1]], _vertex[_index[k+2]] );
        // c = v1 x v2
        cross( _triangle_normal[i], &v1, &v2 );
        // n = c / |c|
        normalize( _triangle_normal[i] );
    }


}

// Translates the normals to the surface (for plotting)
void
Triangles::update_normal_tan_vecs()
{
    for(gint m = 0; m < _tangente.size(); m++)
        plus( _tangente[m] , _vertex[m], _normal[m] );
}


void
Triangles::parse_xml_node(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
{
    cur = cur->childs;
    while (cur != NULL) {
        if( EQSTR(cur->name, "Index") && (cur->ns == ns)) {
            parse_array(_index,
                        (gchar*) xmlNodeListGetString(doc, cur->childs, 1),
                        (gchar*) xmlGetProp(cur, "Length"));
        }
        cur = cur->next;
    }
}


void
Triangles::initialize()
{
    //TODO: fix this!!!
    static guint _loaded = 0;
    ostrstream n;
    n << "triangles" << _loaded++ << ends;
    _name = n.str();
    n.freeze(0);
    //

    _triangle_number = _index.size() / 3;
    _triangle_normal = VectorArray<gfloat>( _triangle_number );
    _tangente = _normal = VectorArray<gfloat>(_vertex.size());
    t_message.generating_normals = TRUE;
    normal_per_triangle();
    normal_per_vertex(90);
    update_normal_tan_vecs();

}


void
Triangles::draw_faces(void)
{
    glDrawElements(GL_TRIANGLES,   _index.size(),
                   GL_UNSIGNED_INT, _index.begin());
}


void
Triangles::draw_edges(void)
{
    glDrawElements(GL_TRIANGLES,   _index.size(),
                   GL_UNSIGNED_INT, _index.begin());
}

void
Triangles::draw_mesh(void)
{
    glDrawElements(GL_TRIANGLES,   _index.size(),
                   GL_UNSIGNED_INT, _index.begin());
}

void
Triangles::draw_lines(void)
{
    glDrawElements(GL_TRIANGLES,   _index.size(),
                   GL_UNSIGNED_INT, _index.begin());
}

void
Triangles::draw_vlines(void)
{
    glDrawElements(GL_TRIANGLES,   _index.size(),
                   GL_UNSIGNED_INT, _index.begin());
}

void
Triangles::draw_hlines(void)
{
    glDrawElements(GL_TRIANGLES,   _index.size(),
                   GL_UNSIGNED_INT, _index.begin());
}

void
Triangles::draw(void)
{

    Object::draw();

    glNormalPointer(GL_FLOAT, 0, _normal.begin());

    //Enable line smooth. Do it here to avoid repetition below.
    glLineWidth (lines_width ());
    if( smooth_lines() ) {
        glEnable (GL_LINE_SMOOTH);
        glEnable (GL_BLEND);
        glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    }


    switch(_polygon_mode )
    {
    case POLY_FILL:
        if( show_mesh() ) {
            glEnable(GL_POLYGON_OFFSET_FILL);
            glPolygonOffset(1.0, 1.0);
        }

        draw_faces();

        /* This will obscure the frame */
        if( smooth_edges() ) {
            glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
            draw_edges();
        }


        if( show_mesh() ) {
            glDisable(GL_POLYGON_OFFSET_FILL);

            // I think this isn't very fast
            // glPushAttrib(GL_LINE_BIT|GL_COLOR_BUFFER_BIT);

            if( mesh_smooth() ) {
                glEnable (GL_LINE_SMOOTH);
                glEnable (GL_BLEND);
                glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
                //glBlendFunc (GL_SRC_ALPHA, GL_ONE);
            }
            else {
                glDisable (GL_LINE_SMOOTH);
                glDisable (GL_BLEND);
            }

            glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );

            { // disable lighting
                glDisableClientState(GL_COLOR_ARRAY);
                glDisable(GL_COLOR_MATERIAL);
                slights.gl_disable_all();
                glDisableClientState(GL_NORMAL_ARRAY);
                // set color
                glColor3fv( & _mesh_color );
            }

            glLineWidth (frame_line_width());

            draw_mesh();

            if( mesh_smooth() && !smooth_lines() ) {
                glDisable (GL_LINE_SMOOTH);
                glDisable (GL_BLEND);

            }
            else if( !mesh_smooth() && smooth_lines() ) {
                glEnable (GL_LINE_SMOOTH);
                glEnable (GL_BLEND);
                glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
            }

            if( slights.some_active() ) {
                glEnableClientState(GL_NORMAL_ARRAY);
                slights.gl_enable_all();
            }
        }

        break;
    case POLY_POINT:
        draw_faces();
        break;
    case POLY_VLINE:
        draw_vlines();
        break;
    case POLY_HLINE:
        draw_hlines();
        break;
    case POLY_LINE:
        draw_lines();
        break;
    default:
        break;
    }

    if( _normals ) {
        { // makes lines white
            glDisableClientState(GL_COLOR_ARRAY);
            glDisable(GL_COLOR_MATERIAL);

            slights.gl_disable_all();

            glDisableClientState(GL_NORMAL_ARRAY);
            glColor3fv( & _normal_color );
        }

        glLineWidth (lines_width ());
        draw_normals();

        if( slights.some_active() ) {
            glEnableClientState(GL_NORMAL_ARRAY);
            slights.gl_enable_all();
        }
    }

    // Disable line smooth
    if( smooth_lines() ) {
        glDisable (GL_LINE_SMOOTH);
        glDisable (GL_BLEND);
    }



}

void
Triangles::draw_normals()
{
    for(gint i = 0; i < _vertex.size(); i++) {
        glBegin(GL_LINES);
        glVertex3fv(_vertex[i]);
        glVertex3fv(_tangente[i]);
        glEnd();
    }
}


