/***************************************************************************
                          util.cc  -  description
                             -------------------
    begin                : Fri Dec 10 1999
    copyright            : (C) 1999 by Juan Pablo Romero Mndez
    email                : juan_pablo_22@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "util.h"
#include "trackball.h"
#include "surface.h"
#include "multiple.h"
#include "read_data.h"

#include <setjmp.h>
#include <glib.h>
#include <math.h>


// #include <gsl_statistics.h>


extern jmp_buf main_loop;
using superficie::minus;
using superficie::plus;


BaseObject*
BASE_OBJECT(GNode* node)
{
    return (node ? ((BaseObject*) node->data) : NULL);
}



gfloat
fit(const gfloat n, const gfloat min, const gfloat max)
{
    return (n - min)/(max - min);
}


template <typename T>
T max(const T * array, const gint length)
{
    T max = array[0];

    for (gint i = 1; i < length; i++)
        if (array[i] > max) max = array[i];

    return max;
}


template <typename T>
T min(const T * array, const gint length)
{
    T min = array[0];

    for (gint i = 1; i < length; i++)
        if (array[i] < min) min = array[i];

    return min;
}


void
generate_default_colors(gfloat* color_array, const float* vertex_array, const gint array_length)
{
    double minval, maxval;
    gint i;
    
    /*
    max = gsl_stats_max( (const gdouble*) vertex_array, array_length);
    min = gsl_stats_min( (const gdouble*) vertex_array, array_length);
    */

    maxval = max( vertex_array, array_length );
    minval = min( vertex_array, array_length );

    for ( i=0; i < array_length; i++ )
        color_array[i] = fit( vertex_array[i], maxval, minval );
}


void
generate_default_colors(VectorArray<gfloat> & color, VectorArray<gfloat> & vertex)
{
    double minval, maxval;

    maxval = max( &vertex, vertex.size() );
    minval = min( &vertex, vertex.size() );

    for (gint i=0; i < vertex.size() * 3; i++ )
        (&color)[i] = fit( (&vertex)[i], maxval, minval );
}


void
cross( gfloat* cross, const gfloat* a, const gfloat* b )
{
    cross[0] = -a[2] * b[1] + a[1] * b[2];
    cross[1] =  a[2] * b[0] - a[0] * b[2];
    cross[2] = -a[1] * b[0] + a[0] * b[1];
}

void
times(gfloat* output, const gfloat* vec, const gfloat escalar)
{
    output[0] = vec[0] * escalar;
    output[1] = vec[1] * escalar;
    output[2] = vec[2] * escalar;
}


void
superficie::plus(gfloat * output, gfloat * vec1, gfloat * vec2)
{
    output[0] = vec1[0] + vec2[0];
    output[1] = vec1[1] + vec2[1];
    output[2] = vec1[2] + vec2[2];
}


void
superficie::minus(gfloat * output, gfloat * vec1, gfloat * vec2)
{
    output[0] = vec1[0] - vec2[0];
    output[1] = vec1[1] - vec2[1];
    output[2] = vec1[2] - vec2[2];
}

/*
gfloat *
util_minus(gfloat * output, gfloat * vec1, gfloat * vec2)
{
    gfloat * out = new gfloat[3];

    out[0] = vec1[0] - vec2[0];
    out[1] = vec1[1] - vec2[1];
    out[2] = vec1[2] - vec2[2];

    return out;
}
*/

void
sum(gfloat* sum, gfloat* vec)
{
    sum[0] += vec[0];
    sum[1] += vec[1];
    sum[2] += vec[2];
}


void
set(gfloat* left_hand, const gfloat* right_hand)
{
    left_hand[0] = right_hand[0];
    left_hand[1] = right_hand[1];
    left_hand[2] = right_hand[2];
}

gint
equal(gfloat* v1, gfloat* v2, gfloat epsilon)
{
    if (fabs(v1[0] - v2[0]) < epsilon &&
        fabs(v1[1] - v2[1]) < epsilon &&
        fabs(v1[2] - v2[2]) < epsilon)
        return TRUE;
    return FALSE;
}

gfloat
distance(gfloat* v1, gfloat* v2)
{
    gfloat tmp[3];

    tmp[0] = v1[0] - v2[0];
    tmp[1] = v1[1] - v2[1];
    tmp[2] = v1[2] - v2[2];

    /* return vlength(tmp); */
    return sqrt(tmp[0] * tmp[0] + tmp[1] * tmp[1] + tmp[2] * tmp[2]);
}        

// Recibe una lista de vectores en R3, pero "aplanados".
void
mean(gfloat* mean, gfloat* vertex, gint vertex_length)
{
    g_assert( vertex );
    gint i;

    vzero(mean);

    for( i = 0; i < vertex_length; i +=3)
        sum(mean, &vertex[i]);

    times(mean, mean, 1/((float) vertex_length / 3));
}


void
normalize(gfloat* vec)
{
    /*
     * if vector has zero length, the normal will also have zero length
     * does this works with opengl?
     */
    times(vec, vec, vlength(vec) ? 1/vlength(vec) : 0 );
}

 void
normal_v(gfloat* normal, gfloat* v1, gfloat* v2, gfloat* v3)
{

    /*

     3
     | \
     |  \
     1---2
     
     normal = (v2 - v1) x (v3 - v1);

     */
    gfloat tmp1[3] = { -v1[0], -v1[1], -v1[2] };
    gfloat tmp2[3];
    gfloat tmp3[3];

    plus(tmp2, v2, tmp1);
    plus(tmp3, v3, tmp1);
    cross( normal, tmp2, tmp3 );

    normalize(normal);
}


/*

gfloat& ind( gfloat* array, gint x, gint y )
{
    return array[ 3 * x + y ];
}


guint& ind( guint* array, gint x, gint y )
{
    return array[ 3 * x + y ];
}
*/


/*
 * this returns the number of the triangle to which
 * the n-th vertex in the index array belongs.
 */
gint
triangle_number(gint n)
{
    return (n / 3);
}

void
weld_vectors(gfloat* vectors, gint vectors_l, gfloat epsilon)
{
    gint n_vec = vectors_l/3;
    gint i, j;
    
        

    for(i = 0; i < n_vec; i++) {
        for(j = i+1; j < n_vec; j++ ) {
            if( distance( &part(vectors, i, 0), &part(vectors, j, 0) ) < epsilon )
                set( &part(vectors, i, 0), &part(vectors, j, 0) );
        }
    }
}


void
calculate_normals_2(gfloat* t_normals,
                    gfloat* normal,
                    guint* t_index,
                    gint t_number)

{
    gint k, i;
    
        
    /*
     * just asign the same normal to the 3 vertices of each triangle
     */
    
    for( k = 0; k < t_number; k++ )
        for( i = 0; i < VECTOR_LENGTH; i++ ) {
            part(normal, part(t_index,k,0), i) = part(t_normals,k,i);
            part(normal, part(t_index,k,1), i) = part(t_normals,k,i);
            part(normal, part(t_index,k,2), i) = part(t_normals,k,i);
        }
}

void
util_farest_vertex(gfloat* vertex, gint length, gfloat* center, gfloat* radius)
{
    gint i;

    

    *radius = distance(vertex, center);
    
    /* Encuentra el vrtice mas alejado del centro */
    for (i = 3; i < length; i += 3) {
        if ( distance(&vertex[i], center) > *radius )
            *radius = distance(&vertex[i], center);
    }
}

/*
void
util_farest_vertex(VectorArray vertex, Vector center, gfloat* radius)
{
    gint i;
    
    *radius = distance(vertex[0], center.ptr());
    for (i = 1; i < length; i ++) {
        if ( distance(vertex[i], center.ptr()) > *radius )
            *radius = distance(vertex[i], center.ptr());
    }
}
*/



gfloat
z_distance(gfloat* vertex, gint length, gfloat* center, gfloat fovy)
{
    gint i;
    gfloat radius;
    gfloat fovyr;

    
    
    util_farest_vertex(vertex, length, center, &radius);

    /* make the radius 20% bigger */
    radius *= 1.2;

    g_message( "radius length = %f", radius);

    /* degrees to radians */
    fovyr = fovy * PI / 180;
        
    /* given  the field of view (fovy), as an angle in radians,
     * calculate the distance betwen the eye and the point marked as the center.
     * (ley de los senos).
     */
    return (radius * sin(PI/2 - fovyr/2) / sin(fovyr/2));
}


FILE * open_file(gchar* name)
{
    FILE* fp = NULL;

    if ((fp = fopen(name, "r")) == NULL) {
        g_warning("can't open %s", name);
        return NULL;
    }
    return fp;
}



/*
 * part(array, x, y) is like array[x][y],
 * and array is 3 x y,
 * but works when array is 1-dimensional
 */

template <typename T>
T & part(T * array, gint x, gint y)
{
    return array[3 * x + y];
}

gboolean
check_index_ok(VectorArray<gfloat> & array, vector<guint> & index)
{
    gfloat max_index = 0;

    max_index = max(index.begin(), index.size());

    if( max_index >= array.size() )
        return FALSE;
    else
        return TRUE;
}


/* Simula la rotacin del objeto sobre un centro arbitrario
 * La magnitud de la rotacin est dada por '_curquat'.
 */

static GLfloat m[4][4] = {
    {0,0,0,0},{0,0,0,0},{0,0,0,0},{0,0,0,0}
};

void
rotate_about(Transformations & tr)
{
    // move to the center of the object
    glTranslatef( tr._center[0],
                  tr._center[1],
                  tr._center[2] );

    // rotation
    build_rotmatrix(m, & tr._curquat);
    glMultMatrixf(&m[0][0]);

    // Back to the original position, but rotated
    glTranslatef( - tr._center[0],
                  - tr._center[1],
                  - tr._center[2] );
}


void
calc_bbox(VectorArray<gfloat> & array, SVector<6> & bbox)
{
    gfloat xext, yext, zext, maxext;
    const gfloat minext = 0.02;

    bbox[0]  = array.max_x();
    bbox[1]  = array.min_x();
    bbox[2]  = array.max_y();
    bbox[3]  = array.min_y();
    bbox[4]  = array.max_z();
    bbox[5]  = array.min_z();

    xext = bbox[0] - bbox[1];
    yext = bbox[2] - bbox[3];
    zext = bbox[4] - bbox[5];

    maxext = xext;
    if( yext > maxext ) maxext = yext;
    if( zext > maxext ) maxext = zext;

    for(gint i=0; i < bbox.size(); i++) {
        if( i % 2 ) // odd
            bbox[i] -= maxext / 20;
        else // even
            bbox[i] += maxext / 20;
    }

}

