## usage:  plot3_sf (X, Y, Z) 
##         plots a 3D curve
##         X, Y, Z must be vectors

## Copyright (C) 1999 Peter Hopfgartner
##
## This file is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2, or (at your option)
## any later version.
##
## This file is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, write to the Free
## Software Foundation, 59 Temple Place - Suite 330, Boston, MA
## 02111-1307, USA.

## Author: Peter Hopfgartner <hopfgartner@rolmail.net>
##
## Created: Dcember 1999

function plot3_sf (X, Y, Z)

  if (nargin == 3)
    if (is_vector(X) && is_vector(Y) && is_vector(Z))
      if ((length(X) != length(Y)) || (length(X) != length(Z)))
	error ("The argument vectors must have the same length");
      endif
    else
      usage ("plot3_sf (X, Y, Z), where X, Y, Z are vectors");
    endif
  else
    usage ("plot_sf (X, Y, Z)");
  endif
    
  xlen = length(X);
  tmp_in_file = tmpnam();
  [fid, msg] = fopen(tmp_in_file, "w");
  if (fid == -1)
    error("while opening superficie input file: \n\t%s", msg);
  endif

  ## Write XML preamble
  fprintf (fid, "<?xml version=\"1.0\"?>\n");
  fprintf (fid, "<sup:Scene xmlns:sup=");
  fprintf (fid, "\"http://www.linuxsupportline.com/~superficie/\">\n");
  fprintf (fid, "  <sup:Objects>\n");
  fprintf (fid, "    <sup:Object Type=\"LINE\">\n");
  fprintf (fid, "      <sup:Common>\n");

  ## Write Vertices
  fprintf (fid, "        <sup:Vertex Length=\"%d\">\n",\
	   3 * xlen);
  for i = 1:xlen
    fprintf (fid, "          %f  %f  %f\n", X(i), Y(i), Z(i));
  endfor
  fprintf (fid, "        </sup:Vertex>\n");
  fprintf (fid, "      </sup:Common>\n");
  fprintf (fid, "    </sup:Object>\n");
  fprintf (fid, "  </sup:Objects>\n");
  fprintf (fid, "</sup:Scene>\n");
  fflush (fid);
  fclose (fid);
  command_string = sprintf('superficie  %s > superficie.log',tmp_in_file);
  system (command_string, 'sync');
  command_string = sprintf("rm %s", tmp_in_file);
  system(command_string);
endfunction