## usage:  quiver3_sf (XX, YY, ZZ, UU, VV, WW) 

## Copyright (C) 1999 Peter Hopfgartner
##
## This file is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2, or (at your option)
## any later version.
##
## This file is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, write to the Free
## Software Foundation, 59 Temple Place - Suite 330, Boston, MA
## 02111-1307, USA.

## Author: Peter Hopfgartner <hopfgartner@rolmail.net>
##
## Created: December 1999

function quiver3_sf (XX, YY, ZZ, UU, VV, WW)


  if (nargin == 3)
    max_vec_len = sqrt(max(max(XX.^2 + YY.^2 + ZZ.^2)));
    UU = XX/max_vec_len;
    VV = YY/max_vec_len;;
    WW = ZZ/max_vec_len;;
    if ((size(UU) != size (VV)) || (size(UU) != size (WW)))
      error ("Matrix dimensions must match");
    endif
    XX = ones(1,rows(UU))'*[1:columns (UU)];
    YY = [1:rows(UU)]'*ones(1,columns (UU));
    ZZ = zeros(UU);
  elseif (nargin == 4)
    max_vec_len = sqrt(max(max(YY.^2 + ZZ.^2 + UU.^2)))
    WW = UU/max_vec_len;
    VV = ZZ/max_vec_len;
    UU = YY/max_vec_len;
    ZZ = XX;
    XX = ones(1,rows(UU))'*[1:columns (UU)];
    YY = [1:rows(UU)]'*ones(1,columns (UU));
  elseif (nargin != 6)
    usage ("quiver3_sf (XX, YY, ZZ, UU, VV, WW) with 3, 4 or 6 arguments");
  endif

  [xrow xcol] = size (XX);

  tmp_in_file = tmpnam();
  [fid, msg] = fopen(tmp_in_file, "w");
  if (fid == -1)
    error("while opening superficie input file: \n\t%s", msg);
  endif

  ## Write XML preamble
  fprintf (fid, "<?xml version=\"1.0\"?>\n");
  fprintf (fid, "<sup:Scene xmlns:sup=");
  fprintf (fid, "\"http://www.linuxsupportline.com/~superficie/\">\n");
  fprintf (fid, "  <sup:Objects>\n");
  fprintf (fid, "    <sup:Object Type=\"LINES\">\n");
  fprintf (fid, "      <sup:Common>\n");

  ## Write Vertices
  fprintf (fid, "        <sup:Vertex Length=\"%d\">\n",\
	   6 * xrow * xcol);
  for i = 1:xrow
    for j = 1:xcol
      fprintf (fid, "          %f  %f  %f  %f  %f  %f\n", \
	       XX(i, j), YY(i, j), ZZ(i, j), \
	       XX(i, j) + UU(i,j), YY(i, j) + VV(i,j), ZZ(i, j) + WW(i,j));
    endfor
  endfor
  fprintf (fid, "        </sup:Vertex>\n");
  fprintf (fid, "      </sup:Common>\n");
  fprintf (fid, "    </sup:Object>\n");
  fprintf (fid, "  </sup:Objects>\n");
  fprintf (fid, "</sup:Scene>\n");
  fflush (fid);
  fclose (fid);
  command_string = sprintf('superficie  %s > superficie.log',tmp_in_file);
  system (command_string, 'sync');
  command_string = sprintf("rm %s", tmp_in_file);
  system(command_string);
endfunction